/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <stdio.h>
#include <gtk/gtk.h>
#include <ctype.h>	//convert char to lower-char
#include <string.h>
#include <stdlib.h>

#include "userdata.h"
#include "confighandling.h"
#include "confighandlingDefinitions.h"
#include "listitems.h"
#include "mytrace.h"
#include "helpers.h"
#include "CreateUsercommandDialog.h"

#include "allInclude.h"

//int openfiles = 0;    //for debuging info. counts the currently opened files, opened with this set of functions.



////////////////////////////////////////////////////////////////////////////////////////////
////////////////XML ConfigFile handling ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
/*
    Some facts:
    the new config file has xml format.
    We will not open the file every time we want some values from it, but we store all the
    information in lists. When we know when we loaded the file the last time and know when
    the file was changed last we can decide to reload the whole file.

    about the xml parsing thing:
    While walking throght the xml file we will come up in the foundstart-callback of the xml parser.
    But we need to know if the found starting string is supported by the state we are currently in.
    With state, we mean e.g. that we currently reading in the properties of the dailog, question or the value and name of an option.
    All this states have valid keywords, others can not be assigned to any property of our internal lists.

    Allright then, the state is stored with the help of _Items_XML_Relation - structure which is stored in
    _Parser_Mode_Struct - structure which helps us to communicate between the callbacks.
    To find out which keywords are valid for our current state we have a function: 'getValidKeyWordsForState'.
    Giving this function the _Items_XML_Relation - structure will give you a pointer to _OptionsList - structure
    where only the gstrName is filled with the valid keywords. We decided to use this structure because many things we
    need (e.g. searching for a name, new item does not need to fill all member variables available) are already implemented.

	The first idea was to search the begining of some blocks before proceeding with the xml parser. But this wasnt a good idea.
	When there are syntax errors in the file the parser gives us a message including the linenumber, but this is never the right line
	as we searched for some string in the file before.
	Further more, when we come to the end string of a block (e.g. CatsEyEBookmarks) the parser reports an error because there is block closed
	which was never opened (in his point of view).

	So, the new method starts loading the whole file, with all information, from the begining of the file.
	From now on it is impossible to load specific blocks such as bookmarks, configurations, commands.
*/


void Usercommand_FoundStartCallback(struct _Parser_Mode_Struct *info);
void Usercommand_FoundEndCallback(struct _Parser_Mode_Struct *info, const char *element_name);
void freeRelicts(struct _Parser_Mode_Struct *info);
void configFileHandling_Init();
int configFileHandling_validateDialogEntityItem (struct _DialogEntityList *dialogItem);
int Usercommands_loadUserCommandsFromFile (struct myFiles_struct *myFilestru, gboolean bMakeItThreadSave);
int Bookmarks_loadBookmarksFromFile (struct myFiles_struct *myFilestru, gboolean bMakeItThreadSave);
int configFileHandling_saveGlobalConfigStructAsXMLConfigFile (FILE *configFile);
int configFileHandling_saveGlobalBookmarkStructAsXMLConfigFile (FILE *configFile);
int configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile (FILE *configFile);




void debugOut_PrintLists();

///////////////////////////////////
//// Some Globals /////////////////

struct _UserCommandList *g_UserCommandList;
struct UserDataList *g_UserDataList;                     //used wthin the contextmenu
struct _OptionsList *g_OptionsList;
struct _OptionsList *g_BookMarkList;
static GString *g_gstrConfigFile;
time_t 	g_timeChanged;  //the configfiles files time stamp
extern GtkWindow 	*win;
static int  g_preventFromReload;    //if this is 1, the list (g_OptionsList, g_BookMarkList and g_UserCommandList)
                                    //will not be deleted, nor reloaded!
                                    //this flag is set and unset primaly in the property page.



///////////////////////////////////
/////////////////////////////////////
//// functions //////////////////

void configFileHandling_Init(){
    TRACEIT (7, "configFileHandling_Init              start");
    //called at startup
    g_UserCommandList = NULL;
    //g_UserCommandsForMarkedItems = NULL;
    g_UserDataList=NULL;
    g_OptionsList = NULL;
    g_BookMarkList = NULL;
    g_gstrConfigFile = NULL;
    g_timeChanged=0;  //the configfiles files time stamp
    g_preventFromReload = 0;

    //set default config file name
    configFileHandling_defineConfigFileName (NULL);

	//g_string_assign(g_gstrConfigFile,"/wurzel/TEMP/test.conf");

    TRACEIT (7, "configFileHandling_Init              end");

}


void configFileHandling_clearForShutdown(){
    TRACEIT (7, "configFileHandling_clearForShutdown              start");
    //frees all memory
    if (NULL != g_gstrConfigFile){
        g_string_free(g_gstrConfigFile, TRUE);
        g_gstrConfigFile = NULL;
    }
    if (g_OptionsList != NULL){
        OptionsList_freeList (&g_OptionsList);
        g_OptionsList = NULL;
    }
    if (g_BookMarkList != NULL){
        OptionsList_freeList (&g_BookMarkList);
        g_BookMarkList = NULL;
    }
    if (g_UserCommandList != NULL){
        UserCommandList_freeList (&g_UserCommandList);
        g_UserCommandList = NULL;
    }
    //if (g_UserCommandsForMarkedItems != NULL){
    //    UserCommandList_freeList (&g_UserCommandsForMarkedItems);
    //    UserCommandList_freeList = NULL;
    //}
    if (g_UserDataList != NULL){
        UserDataList_DeleteList (&g_UserDataList);
    }

    TRACEIT (7, "configFileHandling_clearForShutdown              end");
}


int configFileHandling_defineConfigFileName (const char *ccpFileName){
	//sets g_gstrConfigFile to the given name, thats all.
	//ccpFileName can be NULL; in this case the default name is generated and loaded
	GString *gstrBuf = g_string_sized_new (10);

	if (ccpFileName == NULL){
		//define the default name for the file
		getHomeDirectory (gstrBuf);
		checkPathForLastChar(gstrBuf);
		g_string_append(gstrBuf,CONFIG_FILE_TYPES);
	}
	else{
		g_string_assign (gstrBuf, ccpFileName);
	}

	if (g_gstrConfigFile == NULL){
		g_gstrConfigFile = g_string_new (gstrBuf->str);
	}
	else{
		g_string_assign (g_gstrConfigFile, gstrBuf->str);
	}

	g_string_free (gstrBuf, TRUE);
	return 0;
}


const char *configFileHandling_getConfigFileName (){
    if (g_gstrConfigFile == NULL){
        return NULL;
    }
    return (const char *)g_gstrConfigFile->str;
}


void configFileHandling_setPreventReloadFlag (int iValue){
    TRACEIT (10, "configFileHandling_setPreventReloadFlag          start");
    if (iValue == 0){
        g_preventFromReload = 0;
    }
    else{
        g_preventFromReload = 1;
    }

    TRACEIT (10, "configFileHandling_setPreventReloadFlag          end");
}


int configFileHandling_loadConfigFile(gboolean bIgnoreTimeStamp, gboolean bMakeItThreadSave){
    TRACEIT (7, "configFileHandling_loadConfigFile              start");
    //loads the configfile and stores all information to our UsercommandList g_UserCommandList
    //when the list was filled before, it checks if the configfile has changed meanwhile and then
    //decides wether to reaload the information or not.
    //If bIgnoreTimeStamp is true, the file is loaded in all cases
    //However, if g_preventFromReload is 1, nothing is done, as it is expected that the property page is open which may need the pointer!

    //return: -1=error, 0=ok

    if (g_gstrConfigFile == NULL){
        TRACEIT (2, "configFileHandling_loadConfigFile          end, not correctly initialized");
        return -1;
    }
    if (g_preventFromReload == 1){
        TRACEIT (10, "configFileHandling_loadConfigFile          end, prevent flag set");
        return 0;
    }

    struct myFiles_struct *myFilestru=NULL;
    struct FileAttributs attribs;
    GString *gstrBuf=g_string_sized_new(10);

    //check for the last change
    fileAttributesGet( &attribs, g_gstrConfigFile->str, NULL);
    if (attribs.gstrLinkTarget){
        g_string_free(attribs.gstrLinkTarget, TRUE);
    }

    myFilestru = myFiles_openFile(g_gstrConfigFile->str, "r");
    if (myFilestru == NULL){
	    g_string_free(gstrBuf, TRUE);
        TRACEIT (2, "configFileHandling_loadConfigFile          end, error opening the configfile:");
        TRACEIT (2, g_gstrConfigFile->str);
        return -1;
    }

    //when we come up here we can start to load the file
    if (bIgnoreTimeStamp || g_timeChanged < attribs.utchanged) {
		TRACEIT (7, "configFileHandling_loadConfigFile              start loading file");

		//delete the lists first
		if (g_UserCommandList != NULL){
		    UserCommandList_freeList(&g_UserCommandList);
		    g_UserCommandList = NULL;
		}

	    Options_CreateDefaultList ();   //this will delete the list first

		if (g_BookMarkList != NULL){
		    OptionsList_freeList(&g_BookMarkList);
		    g_BookMarkList = NULL;
		}

		struct _Items_XML_Relation Items_XML_Relation;

		//XML PARSER START

		//our relation struct stores information about where we are in the tree
		XMLRelation_eraseAll(&Items_XML_Relation);

		//our parser struct holds the information where to store the infos from the parser and all that.
		struct _Parser_Mode_Struct modeStruct;
		modeStruct.storageList = NULL;
		modeStruct.relation = &Items_XML_Relation;
		modeStruct.currentListItem = NULL;
		modeStruct.currentOptionBlock=NULL;		//used in Usercommand section
		modeStruct.iCurrentListType = CONFIGFILE_LOAD_NOTHING;	//see enum
		modeStruct.bMakeItThreadSave = bMakeItThreadSave;
		modeStruct.cppAttributeValues = NULL;
		modeStruct.cppAttributeNames = NULL;
		modeStruct.functionStartNotification = NULL;
		modeStruct.functionStopNotification = NULL;
		modeStruct.functionTextNotification = NULL;	//this function handles the Option-Blocks (the only block we use to store information)

//        debugOut_PrintLists();

		XMLParser_ParseFileFile (myFilestru, &modeStruct, bMakeItThreadSave);

		freeRelicts ( &modeStruct );
        g_timeChanged = attribs.utchanged;
    }

    configFileHandling_validateUsercommandList(bMakeItThreadSave);

	g_string_free(gstrBuf, TRUE);
    myFiles_closeFile(myFilestru);

    TRACEIT (10, "configFileHandling_loadConfigFile          end");
    return 0;
}


void configFileHandling_validateUsercommandList(gboolean bMakeItThreadSave){
    // to be a valid usercommand, the following criteria must be true:
    // it must have:
    // gstrName, glistTargets, gstrCommand

    TRACEIT (7, "configFileHandling_validateUsercommandList              start");

    if (g_UserCommandList == NULL){
        TRACEIT (7, "configFileHandling_validateUsercommandList              end, nothing to validate");
        return;
    }

    struct _UserCommandList *usercommandList = g_UserCommandList;
    struct _DialogEntityList *dialogList = NULL;
    int iDialogIsValid;

    while (usercommandList){
        //we first need to validate the dialog entities, if there are some
        dialogList = usercommandList->DialogEntityList;

        iDialogIsValid = 1;
        while (dialogList){
            if (0 == configFileHandling_validateDialogEntityItem(dialogList)){
                iDialogIsValid=0;
                //break;
            }
            dialogList = dialogList->nextItem;
        }
        if (iDialogIsValid == 0){
            usercommandList->iValid = 0;
            usercommandList = usercommandList->nextItem;
            continue;
        }

        //minimum requirements for a command:
        if (usercommandList->gstrName==NULL || usercommandList->glistTargets==NULL || usercommandList->gstrCommand){
            usercommandList->iValid = 0;
            usercommandList = usercommandList->nextItem;
            continue;
        }
        usercommandList->iValid = 1;

        usercommandList = usercommandList->nextItem;
    }
    TRACEIT (7, "configFileHandling_validateUsercommandList              end");
}


int configFileHandling_validateDialogEntityItem (struct _DialogEntityList *dialogItem){
    //return: -1=error, 0=invalid 1=valid
    //only checks one item for validation
    if (dialogItem == NULL){
        TRACEIT (2, "configFileHandling_validateDialogEntityItem              end, parameter NULL");
        return -1;
    }
    int iRet = 0;

    if (dialogItem->iType == -1){
        dialogItem->iValid=0;
        iRet=0;
    }
    if (dialogItem->iType == USERDIALOG_ENTITY_TYPE_QUESTION){
        if ( dialogItem->gstrLabel != NULL && dialogItem->gstrVarIdentifier != NULL ){
            iRet=1;
        }
    }
    else if (dialogItem->iType == USERDIALOG_ENTITY_TYPE_CHECKBOX){
        if ( dialogItem->gstrLabel != NULL && dialogItem->gstrVarIdentifier != NULL ){
            iRet=1;
        }
    }
    else if (dialogItem->iType == USERDIALOG_ENTITY_TYPE_TEXT){
        //gstrLabel can be NULL even if iValid of Object Text is 1, this is because
        //we want to be able to lead an empty text
        //if ( dialogItem->gstrLabel != NULL){
        iRet=1;
        //}
    }
    else if (dialogItem->iType == USERDIALOG_ENTITY_TYPE_FILECHOOSER || dialogItem->iType == USERDIALOG_ENTITY_TYPE_DIRCHOOSER){
        if ( dialogItem->gstrLabel != NULL && dialogItem->gstrVarIdentifier != NULL ){
            iRet=1;
        }
    }
    else if (dialogItem->iType == USERDIALOG_ENTITY_TYPE_SEPARATOR){
        iRet=1;
    }

    if (iRet == 1){
        dialogItem->iValid=1;
    }

    return iRet;
}


int configFileHandling_saveGlobalStructsAsXMLConfigFile (const char *ccpFileName, gboolean bMakeThreadSave){
    /*saves the global structs:
            struct _UserCommandList *g_UserCommandList;
            struct _OptionsList *g_OptionsList;
            struct _OptionsList *g_BookMarkList;
      to the filename specified by ccpFileName
      ccpFileName can be NULL, in this case g_gstrConfigFile is used

      return: 0=ok, -1=error

      we do not check for existance of the file, we just overwrite it
    */
    TRACEIT (10, "configFileHandling_saveGlobalStructsAsXMLConfigFile              start");
    if (ccpFileName == NULL){
        if (g_gstrConfigFile == NULL){
            TRACEIT (3, "configFileHandling_saveGlobalStructsAsXMLConfigFile              parameter NULL");
            return -1;
        }
        ccpFileName = g_gstrConfigFile->str;
    }
    FILE *configFile = fopen (ccpFileName, "w+");
    if (configFile == NULL){
        if (bMakeThreadSave)
            myWrap_gdk_threads_enter("configFileHandling_saveGlobalStructsAsXMLConfigFile");
        MyMessageWidget (win, "error", "Error opening config file for writing.\nPlease check permission and the existance of the path.");
        if (bMakeThreadSave)
            myWrap_gdk_threads_leave("configFileHandling_saveGlobalStructsAsXMLConfigFile");
        TRACEIT (4, "configFileHandling_saveGlobalStructsAsXMLConfigFile              unable to create file");
        return -1;
    }

    fprintf (configFile, "<%s>\n", XMLTAG_MAIN);

    configFileHandling_saveGlobalConfigStructAsXMLConfigFile (configFile);
    configFileHandling_saveGlobalBookmarkStructAsXMLConfigFile (configFile);
    configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile (configFile);

    fprintf (configFile, "</%s>\n", XMLTAG_MAIN);

    fclose (configFile);

    TRACEIT (10, "configFileHandling_saveGlobalStructsAsXMLConfigFile              end");
    return 0;
}


int configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile (FILE *configFile){
    TRACEIT (10, "configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile              start");
    if (configFile == NULL){
        TRACEIT (2, "configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile              parameter NULL");
        return -1;
    }
    if (g_UserCommandList == NULL){
        TRACEIT (10, "configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile              end, no g_UserCommandList defined");
        return -1;
    }
    struct _UserCommandList *UserCommandListBuf = UserCommandList_getFirstItem (g_UserCommandList);
    struct _DialogEntityList *DialogEntityListBuf = NULL;
    GString *gstrBuf = g_string_sized_new (10);
    GList *glistBuf = NULL;

    fprintf (configFile, "<%s>\n", XMLTAG_USERCOMMANDS);
    while (UserCommandListBuf){
        fprintf (configFile, "<%s>\n", XMLTAG_USERCOMMAND);

        //Name
        if (UserCommandListBuf->gstrName){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_NAME, NULL, UserCommandListBuf->gstrName->str);
        }
        //Command
        if (UserCommandListBuf->gstrCommand){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_COMMAND, NULL, UserCommandListBuf->gstrCommand->str);
        }
        //Targets
        if (UserCommandListBuf->glistTargets){
            glistBuf = UserCommandListBuf->glistTargets;
            while (glistBuf){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_TARGET, NULL, ((GString *)glistBuf->data)->str);
                glistBuf = glistBuf->next;
            }
        }
        //DefaultCommand
        if (UserCommandListBuf->iIsDefaultCommand == 1){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_DEFAULTCOMMAND, NULL, "true");
        }
        //Actionnote
        if (UserCommandListBuf->option_gstrActionnote){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_ACTIONNOTE, NULL, UserCommandListBuf->option_gstrActionnote->str);
        }
        //Quitreply
        if (UserCommandListBuf->option_gstrQuitreply){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_QUITREPLY, NULL, UserCommandListBuf->option_gstrQuitreply->str);
        }
        //Calltype
        if (UserCommandListBuf->option_iCallType == USERCOMMAND_CALLTYPE_ALL){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_CALLTYPE, NULL, "all");
        }
        else if (UserCommandListBuf->option_iCallType == USERCOMMAND_CALLTYPE_EACH){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_CALLTYPE, NULL, "each");
        }
        else if (UserCommandListBuf->option_iCallType == USERCOMMAND_CALLTYPE_LIST){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_CALLTYPE, NULL, "list");
        }
        //listWidgets label
        if (UserCommandListBuf->option_gstrLabel){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_LISTLABEL, NULL, UserCommandListBuf->option_gstrLabel->str);
        }
        //listWidgets caption
        if (UserCommandListBuf->option_gstrCaption){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_LISTCAPTION, NULL, UserCommandListBuf->option_gstrCaption->str);
        }
        //Dialog caption
        if (UserCommandListBuf->gstrDialogCaption){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_DIALOGCAPTION, NULL, UserCommandListBuf->gstrDialogCaption->str);
        }
        //Dialog label
        if (UserCommandListBuf->gstrDialogLabel){
            createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_DIALOGLABEL, NULL, UserCommandListBuf->gstrDialogLabel->str);
        }

        //DialogEntities
        DialogEntityListBuf = UserCommandListBuf->DialogEntityList;
        while (DialogEntityListBuf){
            if (DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_QUESTION){
                g_string_append_printf (gstrBuf, "<%s type=\"%s\">", XMLTAG_OBJECT, XMLTAG_QUESTION);
            }
            else if (DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_CHECKBOX){
                g_string_append_printf (gstrBuf, "<%s type=\"%s\">", XMLTAG_OBJECT, XMLTAG_CHECKBOX);
            }
            else if (DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_TEXT){
                g_string_append_printf (gstrBuf, "<%s type=\"%s\">", XMLTAG_OBJECT, XMLTAG_TEXT);
            }
            else if (DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_FILECHOOSER){
                g_string_append_printf (gstrBuf, "<%s type=\"%s\">", XMLTAG_OBJECT, XMLTAG_FILECHOOSER);
            }
            else if (DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_DIRCHOOSER){
                g_string_append_printf (gstrBuf, "<%s type=\"%s\">", XMLTAG_OBJECT, XMLTAG_DIRCHOOSER);
            }
            else if (DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_SEPARATOR){
                g_string_append_printf (gstrBuf, "<%s type=\"%s\">", XMLTAG_OBJECT, XMLTAG_SEPARATOR);
            }
            else {
                TRACEIT (4, "configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile    type of dialog entity unknown in usercommand list, ignoring");
                continue;
            }
            //labelText
            if (DialogEntityListBuf->gstrLabel){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_LABELTEXT, NULL, DialogEntityListBuf->gstrLabel->str );
            }
            //varID
            if (DialogEntityListBuf->gstrVarIdentifier){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_VARID, NULL, DialogEntityListBuf->gstrVarIdentifier->str );
            }
            //predefine
            if (DialogEntityListBuf->gstrPredefine){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_PREDEFINE, NULL, DialogEntityListBuf->gstrPredefine->str );
            }
            //TrueValue
            if (DialogEntityListBuf->gstrAdditional1 && DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_CHECKBOX){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_TRUEVALUE, NULL, DialogEntityListBuf->gstrAdditional1->str );
            }
            //ButtonText
            else if (DialogEntityListBuf->gstrAdditional1 && ( DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_FILECHOOSER || DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_DIRCHOOSER)){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_BUTTONTEXT, NULL, DialogEntityListBuf->gstrAdditional1->str );
            }
            //FalseValue
            if (DialogEntityListBuf->gstrAdditional2 && DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_CHECKBOX){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_FALSEVALUE, NULL, DialogEntityListBuf->gstrAdditional2->str );
            }
            //ChooserCaption
            else if (DialogEntityListBuf->gstrAdditional2 && ( DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_FILECHOOSER || DialogEntityListBuf->iType == USERDIALOG_ENTITY_TYPE_DIRCHOOSER)){
                createUserCommand_createXMLOptionString (gstrBuf, XMLTAG_CHOOSERCAPTION, NULL, DialogEntityListBuf->gstrAdditional2->str );
            }

            g_string_append_printf (gstrBuf, "</%s>\n",XMLTAG_OBJECT);
            DialogEntityListBuf = DialogEntityListBuf->nextItem;
        }
        fprintf (configFile, "%s\n", gstrBuf->str);
        g_string_erase (gstrBuf, 0, -1);

        fprintf (configFile, "</%s>\n", XMLTAG_USERCOMMAND);
        UserCommandListBuf = UserCommandListBuf->nextItem;
    }
    fprintf (configFile, "</%s>\n", XMLTAG_USERCOMMANDS);

    g_string_free (gstrBuf, TRUE);
    TRACEIT (10, "configFileHandling_saveGlobalUsercommandStructAsXMLConfigFile              end");
    return 0;
}


int configFileHandling_saveGlobalBookmarkStructAsXMLConfigFile (FILE *configFile){
    TRACEIT (10, "configFileHandling_saveGlobalBookmarkStructAsXMLConfigFile              start");
    if (configFile == NULL){
        TRACEIT (2, "configFileHandling_saveGlobalBookmarkStructAsXMLConfigFile              parameter NULL");
        return -1;
    }
    if (g_BookMarkList == NULL){
        TRACEIT (10, "configFileHandling_saveGlobalBookmarkStructAsXMLConfigFile              end, no g_BookMarkList defined");
        return -1;
    }
    struct _OptionsList *BookMarkListBuf = OptionsList_getFirstItem (g_BookMarkList);

    GString *gstrBuf = g_string_sized_new (10);

    fprintf (configFile, "<%s>\n", XMLTAG_BOOKMARKS);
    while (BookMarkListBuf){
        createUserCommand_createXMLOptionString (gstrBuf,
                        /*(BookMarkListBuf->gstrID==NULL)?NULL:BookMarkListBuf->gstrID->str,*/
                        NULL,
                        (BookMarkListBuf->gstrName==NULL)?NULL:BookMarkListBuf->gstrName->str,
                        (BookMarkListBuf->gstrValue==NULL)?NULL:BookMarkListBuf->gstrValue->str);
        g_string_append (gstrBuf, "\n");
        BookMarkListBuf = BookMarkListBuf->nextItem;
    }
    fprintf (configFile, "%s</%s>\n", gstrBuf->str, XMLTAG_BOOKMARKS);

    g_string_free (gstrBuf, TRUE);
    TRACEIT (10, "configFileHandling_saveGlobalBookmarkStructAsXMLConfigFile              end");
    return 0;
}


int configFileHandling_saveGlobalConfigStructAsXMLConfigFile (FILE *configFile){
    TRACEIT (10, "configFileHandling_saveGlobalConfigStructAsXMLConfigFile              start");
    if (configFile == NULL){
        TRACEIT (2, "configFileHandling_saveGlobalConfigStructAsXMLConfigFile              parameter NULL");
        return -1;
    }
    if (g_OptionsList == NULL){
        TRACEIT (10, "configFileHandling_saveGlobalConfigStructAsXMLConfigFile              end, no options defined");
        return -1;
    }
    struct _OptionsList *OptionsListBuf = OptionsList_getFirstItem (g_OptionsList);

    GString *gstrBuf = g_string_sized_new (10);

    //XMLTAG_OPTIONS "CatsEyEOptions"

    fprintf (configFile, "<%s>\n", XMLTAG_OPTIONS);
    while (OptionsListBuf){
        createUserCommand_createXMLOptionString (gstrBuf,
                        (OptionsListBuf->gstrID==NULL)?NULL:OptionsListBuf->gstrID->str,
                        (OptionsListBuf->gstrName==NULL)?NULL:OptionsListBuf->gstrName->str,
                        (OptionsListBuf->gstrValue==NULL)?NULL:OptionsListBuf->gstrValue->str);
        g_string_append (gstrBuf, "\n");
        OptionsListBuf = OptionsListBuf->nextItem;
    }
    fprintf (configFile, "%s</%s>\n", gstrBuf->str, XMLTAG_OPTIONS);

    g_string_free (gstrBuf, TRUE);
    TRACEIT (10, "configFileHandling_saveGlobalConfigStructAsXMLConfigFile              end");
    return 0;
}


void debugOut_PrintLists(){
    TRACEIT (5, "debugOut_PrintLists              start");

    printf("--------------------START DEBUG OUT XML FILES ------------------\n");

    if (g_gstrConfigFile){
        printf("file loaded: %s\n",g_gstrConfigFile->str);
    }
    else{
        printf("NO file loaded\n");
    }

    if (g_OptionsList){
        OptionsList_DebugOut_printList(g_OptionsList);
    }
    else{
        printf("g_OptionsList == NULL\n");
    }
    if (g_BookMarkList){
        OptionsList_DebugOut_printList(g_BookMarkList);
    }
    else{
        printf("g_BookMarkList == NULL\n");
    }
    if (g_UserCommandList){
        UserCommandList_DebugOut_printList(g_UserCommandList);
    }
    else{
        printf("g_UserCommandList == NULL\n");
    }

    printf("--------------------END DEBUG OUT XML FILES ------------------\n");

    TRACEIT (5, "debugOut_PrintLists              end");
}


int XMLRelation_eraseAll(struct _Items_XML_Relation *relation){
    TRACEIT (7, "XMLRelation_eraseAll              start");
	//simply writes false to all members to relation

    relation->bUnKnownBlock=FALSE;
	relation->bCatsEyEBlock=FALSE;
	relation->bCatsEyEBookMarksBlock=FALSE;
	relation->bCatsEyEOptionsBlock=FALSE;
	relation->bCatsEyEUserCommandsBlock=FALSE;
    relation->bUserCommandBlock=FALSE;
    relation->bObjectBlock=FALSE;
    relation->bOptionBlock=FALSE;
	relation->bIDBlock=FALSE;
    relation->bNameBlock=FALSE;
    relation->bValueBlock=FALSE;

	relation->iRelation = 0;


    TRACEIT (7, "XMLRelation_eraseAll              end");
	return 0;
}


int XMLRelation_accumulateRelations(struct _Items_XML_Relation *relation){
    TRACEIT (7, "XMLRelation_accumulateRelations              start");
	//return: -1=error, 0=ok
	if (relation == NULL){
		TRACEIT(2, "XMLRelation_accumulateRelations		end, Parameter NULL");
		return -1;
	}
	//we have 19 bools, this makes 19 bits!! How big is an int? We assume that an integer is never less than 20 bytes.
	//in most cases it is 4 bytes which is 32 bits.
	int iRelations=0;

	iRelations |= relation->bUnKnownBlock<<0;				//0 /1
	iRelations |= relation->bCatsEyEBlock<<1;             //1 /2		we are in <CatsEyE> - Block
	iRelations |= relation->bCatsEyEBookMarksBlock<<2;	//2	/4
	iRelations |= relation->bCatsEyEOptionsBlock<<3;      //3	/8		we are in <CatsEyEOptions> - Block
	iRelations |= relation->bCatsEyEUserCommandsBlock<<4; //4 /16
    iRelations |= relation->bUserCommandBlock<<5;			//5	/32		we are in <Usercommand>-Block (within <CatsEyEUsercommands>)
    iRelations |= relation->bObjectBlock<<6;				//6 /64
    iRelations |= relation->bOptionBlock<<7;				//7	/128		we are in <Option> - Block (within <CatsEyEOptions>-Block)
	iRelations |= relation->bIDBlock<<8;					//8 /256
    iRelations |= relation->bNameBlock<<9;				//9 /512
    iRelations |= relation->bValueBlock<<10;				//10 /1024

	relation->iRelation = iRelations;


    TRACEIT (7, "XMLRelation_accumulateRelations              end");
	return 0;
}


void  UserCommandList_DebugOut_printList(struct _UserCommandList *theList){
    TRACEIT (7, "UserCommandList_DebugOut_printList              start");
    if (theList == NULL){
        printf("UserCommandList_DebugOut_printList list: parameter NULL\n");
        return;
    }
    struct _UserCommandList *bufList=UserCommandList_getFirstItem(theList);

    printf("\n------------------START OUTPUT USERCOMMANDS------------------------\n");
    while (bufList){
        printf("____ NEXT ITEM: ____\n");

        printf ("iIsDefaultCommand = %d\n",bufList->iIsDefaultCommand);

        if (bufList->gstrName){
            printf("gstrName: %s\n",bufList->gstrName->str);
        }
        else {
            printf("gstrName not set\n");
        }

        if (bufList->gstrCommand){
            printf("gstrCommand: %s\n",bufList->gstrCommand->str);
        }
        else {
            printf("gstrCommand not set\n");
        }

        if (bufList->gstrDialogCaption){
            printf("gstrDialogCaption: %s\n",bufList->gstrDialogCaption->str);
        }
        else {
            printf("gstrDialogCaption not set\n");
        }

        if (bufList->gstrDialogLabel){
            printf("gstrDialogLabel: %s\n",bufList->gstrDialogLabel->str);
        }
        else {
            printf("gstrDialogLabel not set\n");
        }

        if (bufList->option_gstrQuitreply){
            printf("option_gstrQuitreply: %s\n",bufList->option_gstrQuitreply->str);
        }
        else {
            printf("option_gstrQuitreply not set\n");
        }

        if (bufList->option_gstrActionnote){
            printf("option_gstrActionnote: %s\n",bufList->option_gstrActionnote->str);
        }
        else {
            printf("option_gstrActionnote not set\n");
        }

        if (bufList->option_gstrCaption){
            printf("option_gstrCaption: %s\n",bufList->option_gstrCaption->str);
        }
        else {
            printf("option_gstrCaption not set\n");
        }

        if (bufList->option_gstrLabel){
            printf("option_gstrLabel: %s\n",bufList->option_gstrLabel->str);
        }
        else {
            printf("option_gstrLabel not set\n");
        }

        printf ("option_iCallType = %d\n",bufList->option_iCallType);

        GList *gListBuf = bufList->glistTargets;

        printf("Target List _______________ START\n");
        while (gListBuf){
            printf("Target: %s\n",((GString *)(gListBuf->data))->str);

			gListBuf = gListBuf->next;
		}
        printf("Target List _______________ END\n");


        if (bufList->DialogEntityList){
            DialogEntityList_DebugOut_printList ( bufList->DialogEntityList);
        }
        else{
            printf("No DialogEntities defined\n");
        }

        bufList = bufList->nextItem;
    }
    printf("------------------END OUTPUT USERCOMMANDS------------------------\n\n");


    TRACEIT (7, "UserCommandList_DebugOut_printList              end");
}


void DialogEntityList_DebugOut_printList(struct _DialogEntityList *theList){
    TRACEIT (7, "DialogEntityList_DebugOut_printList              start");
    if (theList == NULL){
        printf("DialogEntityList list: parameter NULL\n");
        return;
    }
    struct _DialogEntityList *bufList=DialogEntityList_getFirstItem(theList);

    printf("\n------------------START OUTPUT DialogEntityList------------------------\n");
    while (bufList){
        printf("____ NEXT ITEM: ____\n");
        printf ("iType = %d\n",bufList->iType);
        printf ("iValid = %d\n",bufList->iValid);


        if (bufList->gstrVarIdentifier){
            printf("gstrVarIdentifier: %s\n",bufList->gstrVarIdentifier->str);
        }
        else {
            printf("gstrVarIdentifier not set\n");
        }

        if (bufList->gstrLabel){
            printf("gstrLabel: %s\n",bufList->gstrLabel->str);
        }
        else {
            printf("gstrLabel not set\n");
        }

        if (bufList->gstrPredefine){
            printf("gstrPredefine: %s\n",bufList->gstrPredefine->str);
        }
        else {
            printf("gstrPredefine not set\n");
        }

        if (bufList->gstrAdditional1){
            printf("gstrAdditional1: %s\n",bufList->gstrAdditional1->str);
        }
        else {
            printf("gstrAdditional1 not set\n");
        }

        if (bufList->gstrAdditional2){
            printf("gstrAdditional2: %s\n",bufList->gstrAdditional2->str);
        }
        else {
            printf("gstrAdditional2 not set\n");
        }

        bufList = bufList->nextItem;
    }
    printf("------------------END OUTPUT DialogEntityList------------------------\n\n");

    TRACEIT (7, "DialogEntityList_DebugOut_printList              end");
}


void OptionsList_DebugOut_printList(struct _OptionsList *theList){
    TRACEIT (7, "OptionsList_DebugOut_printList              start");
    if (theList == NULL){
        printf("Options list: parameter NULL\n");
        return;
    }
    struct _OptionsList *bufList=OptionsList_getFirstItem(theList);

    printf("\n------------------START OUTPUT OPTIONSLIST------------------------\n");
    while (bufList){
        printf("____ NEXT ITEM: ____\n");
        if (bufList->gstrID){
            printf("gstrID: %s\n",bufList->gstrID->str);
        }
        else {
            printf("gstrID not set\n");
        }

        if (bufList->gstrName){
            printf("gstrName: %s\n",bufList->gstrName->str);
        }
        else {
            printf("gstrName not set\n");
        }

        if (bufList->gstrValue){
            printf("gstrValue: %s\n",bufList->gstrValue->str);
        }
        else {
            printf("gstrValue not set\n");
        }
        bufList = bufList->nextItem;
    }
    printf("------------------END OUTPUT OPTIONSLIST------------------------\n\n");

    TRACEIT (7, "OptionsList_DebugOut_printList              end");
}


struct _OptionsList		*XMLParser_getValidKeyWordsForState(struct _Items_XML_Relation *relation){
    TRACEIT (7, "XMLParser_getValidKeyWordsForState              start");
	//returns NULL on error.
	if (relation == NULL){
		TRACEIT(2, "XMLParser_getValidKeyWordsForState		end, Parameter NULL");
		return NULL;
	}

	struct _OptionsList *TagList = NULL;
/*
   	gboolean bUnKnownBlock;				//0 /1
	gboolean bCatsEyEBlock;             //1 /2		we are in <CatsEyE> - Block
	gboolean bCatsEyEBookMarksBlock;	//2	/4
	gboolean bCatsEyEOptionsBlock;      //3	/8		we are in <CatsEyEOptions> - Block
	gboolean bCatsEyEUserCommandsBlock; //4 /16
    gboolean bUserCommandBlock;			//5	/32		we are in <Usercommand>-Block (within <CatsEyEUsercommands>)
    gboolean bObjectBlock;				//6 /64
    gboolean bOptionBlock;				//7	/128		we are in <Option> - Block (within <CatsEyEOptions>-Block)
	gboolean bIDBlock;					//8 /256
    gboolean bNameBlock;				//9 /512
    gboolean bValueBlock;				//10 /1024
*/

	if (relation->bUnKnownBlock==TRUE ){
		TRACEIT(4, "XMLParser_getValidKeyWordsForState		end, not in a valid block (1)");
		return NULL;
	}

	//before we start to fill the list we have to transform the relation structure to somewhat more handy.
	//we do not only need to check for truevalues but also for the false ones to determine the right keywords.
	//I think the easiest way is to bitwise accumulate the bools in the relation structure
	if (0!=XMLRelation_accumulateRelations(relation)){
		TRACEIT(4, "XMLParser_getValidKeyWordsForState		end, error accumulating realtion bools");
		return NULL;
	}
	int iRelation = relation->iRelation;

	//'CatsEyE'
	if (	iRelation == 0 ){
		OptionsList_addItem(&TagList, NULL, XMLTAG_MAIN, NULL); //"CatsEyE"
	}
	//'CatsEyEBookmarks', 'CatsEyEOptions', 'CatsEyEUsercommands'
	if (	iRelation == (2) ){
		OptionsList_addItem(&TagList, NULL, XMLTAG_BOOKMARKS, NULL);    //"CatsEyEBookmarks"
		OptionsList_addItem(&TagList, NULL, XMLTAG_OPTIONS, NULL);    //"CatsEyEOptions"
		OptionsList_addItem(&TagList, NULL, XMLTAG_USERCOMMANDS, NULL);    //"CatsEyEUsercommands"
	}
	//'Usercommand'
	if (	(2 + 16)  == iRelation ){
		OptionsList_addItem(&TagList, NULL, XMLTAG_USERCOMMAND, NULL);   //"Usercommand"
	}
	//'Object'
	if (	(2 + 16 + 32)  == iRelation ){
	    OptionsList_addItem(&TagList, NULL, XMLTAG_OBJECT, NULL);    //"Object"
	}
	//'Option'
	if (	(2 + 4)  == iRelation ||
			(2 + 8)  == iRelation ||
			(2 + 16 + 32) == iRelation ||
			(2 + 16 + 32 + 64) == iRelation
		){
		OptionsList_addItem(&TagList, NULL, XMLTAG_OPTION, NULL);    //"Option"
	}
	//'ID'
	if (	(2 + 4 + 128) == iRelation ||
			(2 + 8 + 128) == iRelation ||
			(2 + 16 + 32 + 128) == iRelation ||
			//(iRelation & (2 + 16 + 32 + 64) ) == iRelation ||
			(2 + 16 + 32 + 64 + 128) == iRelation
		){
		OptionsList_addItem(&TagList, NULL, XMLTAG_ID, NULL);    //"ID"
	}

	//'Name', 'Value'
	if (	(2 + 4 + 128) == iRelation ||
			(2 + 8 + 128) == iRelation ||
			(2 + 16 + 32 + 128) == iRelation ||
			(2 + 16 + 32 + 64 + 128) == iRelation
		){
		OptionsList_addItem(&TagList, NULL, XMLTAG_NAME, NULL);     //"Name"
		OptionsList_addItem(&TagList, NULL, XMLTAG_VALUE, NULL);    //"Value"
	}


    TRACEIT (7, "XMLParser_getValidKeyWordsForState              end");
    if (TagList)
        return OptionsList_getFirstItem(TagList);
    else
        return NULL;
}


void Usercommand_FoundStartCallback(struct _Parser_Mode_Struct *info){
    TRACEIT (7, "Usercommand_FoundStartCallback              start");
	//when we come up here we know, that the current state is valid (stored in info->relation.
    //here we do only react on the current state. This means we reserver memory when needed.
	//this function us used for <CatsEyEUsercommands> which includes <Usercommand>,<Option> and <Object>

    //Each time we found a <Usercommand>-Block we reserve memory for one item (_UserCommandList), the pointer to
    //this structure (which is not yet added to the storageList) is saved in currentListItem

	//When we find an Object-Block we reserve the memory for _DialogEntityList in currentListItem->DialogEntityList (_UserCommandList)

	//if we find an <Option>-Block we reserve memory for one item (_OptionsList) and store
	//the pointer in modeStruct.currentOptionBlock to collect the data
	if (info == NULL){
		TRACEIT (2, "Usercommand_FoundStartCallback			end, parameter NULL");
		return;
	}

	const char **cppAttribNames = info->cppAttributeValues;
	const char **cppAttribValues = info->cppAttributeNames;
	//GString *gstrBuf=g_string_sized_new(10);
	const char *cpType=NULL;
	int iType = -1;

	//<Usercommand>-Block
    if (	info->relation->iRelation == (2 + 16 + 32)
    ){
        if (info->currentListItem != NULL){ //this is some Error, we try to free it
            TRACEIT (2, "Usercommand_FoundStartCallback         Error, info->currentListItem isnt NULL, trying to free it");
            //free(info->currentListItem);
            UserCommandList_freeList((struct _UserCommandList **)&info->currentListItem);
        }
        UserCommandList_addItem((struct _UserCommandList **)&info->currentListItem, "Usercommand_FoundStartCallback");
    }

   	//<Option>-Block
    else if (	info->relation->iRelation == (2 + 16 + 32 + 128) ||
    			info->relation->iRelation == (2 + 16 + 32 + 64 + 128)
    ){
    	if (info->currentOptionBlock != NULL){ //this is some Error, we try to free it
            TRACEIT (2, "Usercommand_FoundStartCallback         Error, info->currentOptionBlock isnt NULL, trying to free it");
            OptionsList_freeList (&(info->currentOptionBlock));
        }
        OptionsList_addItem (&(info->currentOptionBlock), NULL, NULL, NULL);
    }

    //<Object>-Block
    else if (	info->relation->iRelation == (2 + 16 + 32 + 64)
    ){
    	if (info->currentListItem == NULL){
    		TRACEIT (4, "Usercommand_FoundStartCallback         Error, info->currentListItem is NULL, OBJECT block was unsuspected");
    	}
    	else if (cppAttribNames == NULL || cppAttribValues == NULL){
    		TRACEIT (4, "Usercommand_FoundStartCallback         Error,No attributes defined in ObjectBlock, define attribute type");
    	}
    	else{
    		//we now search for the type of the object (qestion, checkbox, text, file/dirchooser)
    		while (*cppAttribValues != NULL){
    			//if we want to have caseinsensitive comparision at this point uncomment the following lines
    			//but mind, that stcmp(...) has to compare the right strings
    			//g_string_assign(gstrBuf, cppAttribNames);
				//cpBuf = g_utf8_strdown( gstrBuf->str,-1);
				//if (cpBuf){
				//	g_string_assign(gstrBuf,  cpBuf);
				//	g_free(cpBuf);
				//}
    			if (0==strcmp (*cppAttribValues, XMLTAG_TYPE)){      //"type"
    				cpType = *cppAttribNames;
    			}
   				cppAttribNames++;
   				cppAttribValues++;
    		}
    		if (cpType){
    			if (0==strcmp (cpType, XMLTAG_QUESTION) ){     //"question"
    				iType = USERDIALOG_ENTITY_TYPE_QUESTION;
    			}
    			else if (0==strcmp (cpType, XMLTAG_CHECKBOX) ){        //"checkbox"
    				iType = USERDIALOG_ENTITY_TYPE_CHECKBOX;
    			}
    			else if (0==strcmp (cpType, XMLTAG_TEXT) ){        //"text"
    				iType = USERDIALOG_ENTITY_TYPE_TEXT;
    			}
    			else if (0==strcmp (cpType, XMLTAG_FILECHOOSER) ){    //"filechooser"
    				iType = USERDIALOG_ENTITY_TYPE_FILECHOOSER;
    			}
    			else if (0==strcmp (cpType, XMLTAG_DIRCHOOSER) ){    //"dirchooser"
    				iType = USERDIALOG_ENTITY_TYPE_DIRCHOOSER;
    			}
    			else if (0==strcmp (cpType, XMLTAG_SEPARATOR) ){        //"separator"
    				iType = USERDIALOG_ENTITY_TYPE_SEPARATOR;
    			}
    		}
    		DialogEntityList_addItem( &(((struct _UserCommandList *)(info->currentListItem))->DialogEntityList), iType );
    	}
    }

    TRACEIT (7, "Usercommand_FoundStartCallback              end");
    return;
}


void Usercommand_FoundEndCallback(struct _Parser_Mode_Struct *info, const char *element_name){
    TRACEIT (7, "Usercommand_FoundEndCallback              start");
	//When comming from <Object> we have nothing to do, as all the information is allready stored in _usercommandList->Dialog..

	//when comming from an <Option> Block, we can be in <UserCommand> or <Object>
	//and we have to add the information in info->currentOptionBlock to info->currentListItem (_UserCommandList)
	//or info->currentListItem->DialogEntityList (_DialogEntityList), then, FREE the currentOptionBlock!! and set the pointer to NULL

	//when we leave <UserCommand> copy the currentItem to the storageList (do not copy the content readress the next/Prev members)
	//and set the currentItem to NULL

	//we also can be in an Option block, with ID, Name and Value properties, but this doesnt bother us in here.

	if (info == NULL || element_name == NULL){
		TRACEIT (2, "Usercommand_FoundEndCallback			end, parameter NULL");
		return;
	}
	if (info->storageList == NULL || info->currentListItem==NULL || info->relation==NULL ){
	    /*if (info->storageList == NULL ){
            printf("storageList == NULL\n");
	    }
	    if (info->currentListItem == NULL ){
            printf("currentListItem == NULL\n");
	    }
	    if (info->relation == NULL ){
            printf("relation == NULL\n");
	    }*/
		TRACEIT(2, "Usercommand_FoundEndCallback			end, parameter 2 NULL");
		return;
	}

	int iRelation = info->relation->iRelation;
	struct _UserCommandList *currentListItem = (struct _UserCommandList *)info->currentListItem;
	struct _OptionsList *currentOptionBlock = info->currentOptionBlock;
	char *cbuf=NULL;

	if (0 == strcmp(element_name, XMLTAG_OBJECT)){	//nothing to do, but lets check if the option block has quit
		if (info->currentOptionBlock != NULL){	//this should not happen, however, the info stored in here is not used, we free it
			TRACEIT (4, "Usercommand_FoundEndCallback			leaving Object block, but an Option block is still active");
			free(info->currentOptionBlock);
			info->currentOptionBlock = NULL;
		}
		//return;
	}
	else if ( 	( iRelation==(0) || iRelation==(2) || iRelation==(2+16) )  &&
				( 0==strcmp(element_name, XMLTAG_MAIN) || 0==strcmp(element_name, XMLTAG_USERCOMMANDS) || 0==strcmp(element_name, XMLTAG_USERCOMMAND) )
	){	//we save the current state of the usercommand to our list.
	    if (info->currentListItem != NULL){
            UserCommandList_addItemAsCopy((struct _UserCommandList **)info->storageList, currentListItem);
            info->currentListItem = NULL;
            if (info->currentOptionBlock != NULL){
                //now free the option list
                OptionsList_freeList ( &(info->currentOptionBlock) );
                info->currentOptionBlock=NULL;
            }
		}
	}
	else if (0 == strcmp(element_name, XMLTAG_OPTION)){
		if ( info->currentOptionBlock == NULL){
			TRACEIT (4, "Usercommand_FoundEndCallback			end, empty Option block? Can not continue");
			return;
		}
		if (info->currentOptionBlock->gstrID == NULL){
			TRACEIT (4, "Usercommand_FoundEndCallback			end, No ID defined in option block");
			OptionsList_freeList ( &(info->currentOptionBlock) );
			info->currentOptionBlock=NULL;
			return;
		}
		if (info->currentOptionBlock->gstrValue == NULL){
			TRACEIT (4, "Usercommand_FoundEndCallback			end, No value defined in option block");
			OptionsList_freeList ( &(info->currentOptionBlock) );
			info->currentOptionBlock=NULL;
			return;
		}
		if 	( iRelation == (2 + 16 + 32 ) ){ 	//usercommand block, store currentOption to currentListItem
			//we check for the ID (stored in currentOption) for valid names to identify the right varible to store the information to
			if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_DEFAULTCOMMAND)){   //"Defaultcommand"
				cbuf = currentOptionBlock->gstrValue->str;
				if ( 0==strcmp(cbuf, "True") || 0==strcmp(cbuf, "true") || 0==strcmp(cbuf, "TRUE") ){
					currentListItem->iIsDefaultCommand = 1;
				}
				else{
					currentListItem->iIsDefaultCommand = 0;
				}
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_NAME)){
				UserCommandList_editItem(currentListItem, -1, currentOptionBlock->gstrValue->str, NULL, NULL, -1, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_TARGET)){  //"Target"
				 UserCommandList_editItem(currentListItem, -1, NULL, currentOptionBlock->gstrValue->str, NULL, -1, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_COMMAND)){  //"Command"
				UserCommandList_editItem(currentListItem, -1, NULL, NULL, currentOptionBlock->gstrValue->str, -1, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_CALLTYPE)){      //"Calltype"
				cbuf = currentOptionBlock->gstrValue->str;
				if ( 0==strcmp(cbuf, "all") || 0==strcmp(cbuf, "All") || 0==strcmp(cbuf, "ALL") ){
					UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, USERCOMMAND_CALLTYPE_ALL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
				}
				else if ( 0==strcmp(cbuf, "each") || 0==strcmp(cbuf, "Each") || 0==strcmp(cbuf, "EACH") ){
					UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, USERCOMMAND_CALLTYPE_EACH, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
				}
				else if ( 0==strcmp(cbuf, "list") || 0==strcmp(cbuf, "List") || 0==strcmp(cbuf, "LIST") ){
					UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, USERCOMMAND_CALLTYPE_LIST, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
				}
				else {
                    TRACEIT (4, "Usercommand_FoundEndCallback              unknown calltype, expected all, each or list");
                    TRACEIT (4, "But got:");
                    TRACEIT (4, cbuf);
                }
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_QUITREPLY)){      //"OptionQuitreply"
				UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, -1, currentOptionBlock->gstrValue->str, NULL, NULL, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_ACTIONNOTE)){  //"OptionActionnote"
				UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, -1, NULL, currentOptionBlock->gstrValue->str, NULL, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_LISTCAPTION)){      //"OptionCaption"
				UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, -1, NULL, NULL, currentOptionBlock->gstrValue->str, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_LISTLABEL)){  //"OptionLabel"
				UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, -1, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_DIALOGCAPTION)){   //"DialogCaption"
			    UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, -1, NULL, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_DIALOGLABEL)){      //"DialogLabel"
			    UserCommandList_editItem(currentListItem, -1, NULL, NULL, NULL, -1, NULL, NULL, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL);
			}
			else {
                TRACEIT (4, "Usercommand_FoundEndCallback              unknown ID for 2 + 16 + 32, expected OptionQuitreply, OptionActionnote, OptionCaption, OptionLabel");
                TRACEIT (4, "But got:");
                TRACEIT (4, currentOptionBlock->gstrID->str);
			}
		}
		else if	( iRelation == (2 + 16 + 32 + 64 ) ){	//object block, store currentOption to currentListItem->DialogEntityList
			if ( ((struct _UserCommandList *)(info->currentListItem))->DialogEntityList == NULL){
				TRACEIT (2, "Usercommand_FoundEndCallback				end, Dialog pointer is NULL but we are in a dialog section, weird!");
				return;
			}
			if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_VARID)){   //"VarID"
				DialogEntityList_editLastItem(currentListItem->DialogEntityList, -1, currentOptionBlock->gstrValue->str, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_LABELTEXT)){  //"LabelText"
				DialogEntityList_editLastItem(currentListItem->DialogEntityList, -1, NULL, currentOptionBlock->gstrValue->str, NULL, NULL, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_PREDEFINE)){  //"Predefine"
				DialogEntityList_editLastItem(currentListItem->DialogEntityList, -1, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL, NULL, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_TRUEVALUE)){  //"TrueValue"
				DialogEntityList_editLastItem(currentListItem->DialogEntityList, -1, NULL, NULL, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_BUTTONTEXT)){ //"ButtonText"
				DialogEntityList_editLastItem(currentListItem->DialogEntityList, -1, NULL, NULL, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_FALSEVALUE)){  //"FalseValue"
				DialogEntityList_editLastItem(currentListItem->DialogEntityList, -1, NULL, NULL, NULL, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL);
			}
			else if ( 0 == strcmp(currentOptionBlock->gstrID->str, XMLTAG_CHOOSERCAPTION)){  //"ChooserCaption"
				DialogEntityList_editLastItem(currentListItem->DialogEntityList, -1, NULL, NULL, NULL, NULL, NULL, NULL, currentOptionBlock->gstrValue->str, NULL);
			}
			else {
                TRACEIT (4, "Usercommand_FoundEndCallback              unknown ID for 2 + 16 + 32 + 64: expected: VarID, LabelText, Predefine, TrueValue, ButtonText, FalseValue or ChooserCaption");
                TRACEIT (4, "But got:");
                TRACEIT (4, currentOptionBlock->gstrID->str);
			}
		}
		if (info->currentOptionBlock != NULL){
			//now free the option list
			OptionsList_freeList ( &(info->currentOptionBlock) );
			info->currentOptionBlock=NULL;
		}
	}
    TRACEIT (7, "Usercommand_FoundEndCallback              end");
}


void Options_FoundStartCallback(struct _Parser_Mode_Struct *info){
    TRACEIT (7, "Options_FoundStartCallback              start");
    //when we come up here we know, that the current state is valid (stored in info->relation.
    //here we do only react on the current state. This means we reserver memory when needed.
	//this function us used for <CatsEyEOptions> and <CatsEyEBookmarks>

    //Each time we found a <Option>-Block we reserve memory for one item (_Options_List), the pointer to
    //this structure (which is not yet added to the storageList) is saved in currentListItem
	if (info == NULL){
		TRACEIT (2, "Options_FoundStartCallback			end, parameter NULL");
		return;
	}

    if (info->relation->iRelation == (2 + 8 + 128) ||	//options block
    	info->relation->iRelation == (2 + 4 + 128) 		//bookmark block
    ){
        if (info->currentListItem != NULL){ //this is some Error, we try to free it
            TRACEIT (2, "Options_FoundStartCallback         Error, info->currentListItem isnt NULL, trying to free it");
            OptionsList_freeList ((struct _OptionsList **)&(info->currentListItem));
        }
        OptionsList_addItem ( (struct _OptionsList **)&(info->currentListItem), NULL, NULL, NULL);
        //printf("Options_FoundStartCalback: Item added to OptionsList\n");
    }
    //printf("Options_FoundStartCallback, ends: info->relation->iRelation = %d\n",info->relation->iRelation);
    TRACEIT (7, "Options_FoundStartCallback              end");
}


void Options_FoundEndCallback(struct _Parser_Mode_Struct *info, const char *element_name){
    TRACEIT (7, "Options_FoundEndCallback              start");
    //when info->currentListItem isnt NULL, it is suspected to be a valid item for the options list
    //WE DELETE the currentListItem in here.
    //However, we have to check its validation before adding it to the list, as some options may need
    //defined information or may not be allowed to exist more than once.

	if (info == NULL || element_name==NULL){
		TRACEIT (2, "Options_FoundEndCallback         end, Parameter NULL");
		return;
	}
	if (info->relation == NULL || info->storageList == NULL || info->currentListItem==NULL ){
	    if (info->relation == NULL){
            TRACEIT (2, "Options_FoundEndCallback   relation==0");
	    }
	    if (info->storageList == NULL){
            TRACEIT (2, "Options_FoundEndCallback   storageList==0");
	    }
	    if (info->currentListItem == NULL){
            TRACEIT (2, "Options_FoundEndCallback   currentListItem==0");
	    }

		if (info->currentListItem!=NULL){
			OptionsList_freeList( (struct _OptionsList **)&(info->currentListItem) );
		}
		TRACEIT (2, "Options_FoundEndCallback         end, Parameter  2 NULL");
		return;
	}
	struct _OptionsList *currentListItem = (struct _OptionsList *)info->currentListItem;
	struct _OptionsList **OptionsList = (struct _OptionsList **)info->storageList;
	struct _OptionsList *BufObject=NULL;

	int SingleOrMulti=0;	//0=unknown/notfound/invalid, 1=found/single, 2=found/multi		//multi means that the key can exists multiple times

	if (info->relation->iRelation == (2 + 8) ){	//supposed to leav option block from options block
		//check if the ID is known:
		if (0 == strcmp (STANDARDEDITORKEY, currentListItem->gstrID->str) && currentListItem->gstrName!=NULL && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (SAVEPATHESONEXIT, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (TERMINALKEY, currentListItem->gstrID->str) && currentListItem->gstrName!=NULL && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (STANDARDSTARTDIRECTORY, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (COLOR_ITEMS_DEFAULT, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (COLOR_DIRECTORIES, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (COLOR_LINKS, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (COLOR_ITEMS_MISSING, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=1;
		}
		else if (0 == strcmp (STARTDIRECTORIESLEFTKEY, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=2;
		}
		else if (0 == strcmp (STARTDIRECTORIESLEFTKEY, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
			SingleOrMulti=2;
		}
		else if (0 == strcmp (PREVIEWSIZE, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
            SingleOrMulti=1;
		}
		else if (0 == strcmp (PREVIEWTYPES, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
            SingleOrMulti=1;
		}
		else if (0 == strcmp (SAVESESSIONMINUTESTYPES, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
            SingleOrMulti=1;
		}
		else if (0 == strcmp (COLUMN_ORDER, currentListItem->gstrID->str) && currentListItem->gstrValue!=NULL){
            SingleOrMulti=1;
		}
		else{
			TRACEIT (5, "Options_FoundEndCallback         unknown parameter or incomplete values");
			TRACEIT (5, "Element Name:");
			TRACEIT (5, element_name);
			if (currentListItem->gstrID){
			    TRACEIT (5, "currentListItem->gstrID:");
                TRACEIT (5, currentListItem->gstrID->str);
			}
			else{
			    TRACEIT (5, "No ID defined");
			}
			if (currentListItem->gstrValue){
			    TRACEIT (5, "currentListItem->gstrValue:");
                TRACEIT (5, currentListItem->gstrValue->str);
			}
			else{
			    TRACEIT (5, "No Value defined");
			}
		}
	}
	else if (info->relation->iRelation == (2 + 4) ){	//supposed to leaved option block from Bookmarks block
		if (currentListItem->gstrName != NULL && currentListItem->gstrValue != NULL){
			SingleOrMulti=2;
		}
	}

	if (SingleOrMulti == 1){
		BufObject = OptionsList_getItemByID(*OptionsList, currentListItem->gstrID->str);
		if (BufObject == NULL){
			OptionsList_addItem(OptionsList, currentListItem->gstrID==NULL?NULL:currentListItem->gstrID->str,
								currentListItem->gstrName==NULL?NULL:currentListItem->gstrName->str,
								currentListItem->gstrValue==NULL?NULL:currentListItem->gstrValue->str);
		}
		else{
            OptionsList_editItem(BufObject, currentListItem->gstrID==NULL?NULL:currentListItem->gstrID->str,
								currentListItem->gstrName==NULL?NULL:currentListItem->gstrName->str,
								currentListItem->gstrValue==NULL?NULL:currentListItem->gstrValue->str);
			//TRACEIT (5, "Options_FoundEndCallback         found another instance of a single option, last wins");
		}
		OptionsList_freeList( (struct _OptionsList **)&(info->currentListItem) );
        info->currentListItem = NULL;
	}
	else if (SingleOrMulti == 2){
	    //printf("Options_FoundEndCallback multi2 adding item\n");
		OptionsList_addItem(OptionsList, currentListItem->gstrID==NULL?NULL:currentListItem->gstrID->str,
								currentListItem->gstrName==NULL?NULL:currentListItem->gstrName->str,
								currentListItem->gstrValue==NULL?NULL:currentListItem->gstrValue->str);
        OptionsList_freeList( (struct _OptionsList **)&(info->currentListItem) );
        info->currentListItem = NULL;
	}

    TRACEIT (7, "Options_FoundEndCallback              end");
}


void Options_FoundTextCallback(struct _Parser_Mode_Struct *info, const char *text){
    TRACEIT (7, "Options_FoundTextCallback              start");
	//We do not longer only handle options and bookmarks in here but all <Option> Blocks we use
	//including the Option blocks in the usercommand/dialog sections

	//this means, we only check for 256m 512 or 1024 in iRelation (wich holds the information about where we are)
	//and react to that.

	//However, option and bookmark block store their OptionsList in currentListItem, while
	//the commandblock store it in currentOptionBlock, we distinguish this with the help of
	//iCurrentListType


	//no:
	//////we are here for the options and bookmarks, so the only valid relation modes are:
	//////( 2 + 8 + 128 + 256 )		( 2 + 8 + 128 + 512 )	( 2 + 8 + 128 + 1024 )

	if (info == NULL || text == NULL){
		TRACEIT (2, "Options_FoundTextCallback           end, Parameter NULL");
        return;
	}

	struct _Items_XML_Relation *relation = (struct _Items_XML_Relation *)info->relation;
    if (info->relation==NULL){
        TRACEIT (2, "Options_FoundTextCallback           end, Parameter 2 NULL");
        return;
    }
    struct _OptionsList	*optionList = NULL;
    if (info->iCurrentListType == CONFIGFILE_LOAD_OPTIONS || info->iCurrentListType==CONFIGFILE_LOAD_BOOKMARKS){
    	optionList = (struct _OptionsList *)info->currentListItem;
    }
    else if (info->iCurrentListType == CONFIGFILE_LOAD_USERCOMMANDS){
    	optionList = info->currentOptionBlock;
    }

    if (optionList == NULL){    //we can come up here when there is text (e.g. a newline) where we do not prepared for text
        /*
    	TRACEIT (2, "Options_FoundTextCallback			end, Parameter 3 NULL");
    	if (info->iCurrentListType == CONFIGFILE_LOAD_OPTIONS){
            TRACEIT(2, "Options_FoundTextCallback comming from CONFIGFILE_LOAD_OPTIONS");
    	}
    	if (info->iCurrentListType == CONFIGFILE_LOAD_BOOKMARKS){
            TRACEIT(2, "Options_FoundTextCallback comming from CONFIGFILE_LOAD_BOOKMARKS");
    	}
    	if (info->iCurrentListType == CONFIGFILE_LOAD_USERCOMMANDS){
            TRACEIT(2, "Options_FoundTextCallback comming from CONFIGFILE_LOAD_USERCOMMANDS");
    	}*/
    	TRACEIT (7, "Options_FoundTextCallback              end 2");
    	return;
    }

	if ( 256  & relation->iRelation ){	//ID
	    TRACEIT (6, "Options_FoundTextCallback			editItem at optionList ID:");
	    TRACEIT (6, text);
		OptionsList_editItem( optionList, text, NULL, NULL);
	}
	if ( 512 & relation->iRelation ){	//Name
	    TRACEIT (6, "Options_FoundTextCallback			editItem at optionList Name:");
	    TRACEIT (6, text);
		OptionsList_editItem( optionList, NULL, text, NULL);
	}
	if ( 1024 & relation->iRelation ){  //Value
	    TRACEIT (6, "Options_FoundTextCallback			editItem at optionList Value:");
	    TRACEIT (6, text);
		OptionsList_editItem( optionList, NULL, NULL, text);
	}

    TRACEIT (7, "Options_FoundTextCallback              end");
}


int UserCommandList_addItem(struct _UserCommandList **theList, const char *ccpSource){
    TRACEIT (7, "UserCommandList_addItem              start");
	//return: -1=error, 0=ok

	if (theList == NULL){
		TRACEIT (2, "UserCommandList_addItem			end, parameter NULL");
		return -1;
	}

	struct _UserCommandList *newItem = malloc (sizeof (struct _UserCommandList));
	struct _UserCommandList *bufItem = NULL;
	if (newItem == NULL){
		TRACEIT (2, "UserCommandList_addItem			Could not allocate memory");
		return -1;
	}
	/*if (ccpSource != NULL){
	    char cbuf[301];
        snprintf(cbuf, 300,"%s added new item:%p",ccpSource, newItem);
        TRACEIT (4, cbuf);
	}*/


	//default values for member
	newItem->iIsDefaultCommand 		= 0;
	newItem->iValid                 = 0;
	newItem->gstrName				=NULL;
	newItem->glistTargets			=NULL;
	newItem->gstrCommand			=NULL;
	newItem->DialogEntityList		=NULL;
	newItem->gstrDialogCaption		=NULL;
	newItem->gstrDialogLabel		=NULL;
	newItem->option_iCallType		=USERCOMMAND_CALLTYPE_ALL;
	newItem->option_gstrQuitreply	=NULL;
	newItem->option_gstrActionnote	=NULL;
	newItem->option_gstrCaption		=NULL;
	newItem->option_gstrLabel		=NULL;
	newItem->gstrWorkingDir		    =NULL;

	newItem->nextItem = NULL;
	newItem->prevItem = NULL;

	//store the new item
	if (*theList == NULL){	//brand new list
		*theList = newItem;
	}
	else{
		bufItem = UserCommandList_getLastItem (*theList);
		if (bufItem == NULL){
			TRACEIT (2, "UserCommandList_addItem			error, its no new list, but I also cant find the last item");
			free(newItem);
			return -1;
		}

		bufItem->nextItem = newItem;
		newItem->prevItem = bufItem;
	}

    TRACEIT (7, "UserCommandList_addItem              end");
	return 0;
}


int UserCommandList_clearTargets(struct _UserCommandList *theItem){
    TRACEIT (10, "UserCommandList_clearTargets              start");
    if (theItem == NULL ){
        TRACEIT (2, "UserCommandList_clearTargets        end, PARAMETER NULL");
        return -1;
    }
    if (theItem->glistTargets){
        g_list_foreach(theItem->glistTargets, (GFunc) GListFreeGStrings, NULL);
        g_list_free(theItem->glistTargets);
    }
    theItem->glistTargets = NULL;
    TRACEIT (10, "UserCommandList_clearTargets              end");
    return 0;
}


struct _UserCommandList *UserCommandList_deleteItem(struct _UserCommandList *theItem){
    //deletes theItem and returns the new begining of the list,
    //if theItem was the last item NULL is returned
    TRACEIT (7, "UserCommandList_deleteItem              start");
    struct _UserCommandList *bufList = NULL;
    if (theItem == NULL ){
        TRACEIT (2, "UserCommandList_deleteItem        end, PARAMETER NULL");
        return NULL;
    }
    if (theItem->nextItem){
        theItem->nextItem->prevItem = theItem->prevItem;
        bufList = theItem->nextItem;
    }
    if (theItem->prevItem){
        theItem->prevItem->nextItem = theItem->nextItem;
        bufList = theItem->prevItem;
    }

    if (bufList){
        bufList = UserCommandList_getFirstItem (bufList);
    }


    if (theItem->gstrName){
        g_string_free(theItem->gstrName, TRUE);
    }
    if (theItem->glistTargets){
        g_list_foreach(theItem->glistTargets, (GFunc) GListFreeGStrings, NULL);
        g_list_free(theItem->glistTargets);
    }
    if (theItem->gstrCommand){
        g_string_free(theItem->gstrCommand, TRUE);
    }
    if (theItem->gstrDialogCaption){
        g_string_free(theItem->gstrDialogCaption, TRUE);
    }
    if (theItem->gstrDialogLabel){
        g_string_free(theItem->gstrDialogLabel, TRUE);
    }
    if (theItem->option_gstrQuitreply){
        g_string_free(theItem->option_gstrQuitreply, TRUE);
    }
    if (theItem->option_gstrActionnote){
        g_string_free(theItem->option_gstrActionnote, TRUE);
    }
    if (theItem->option_gstrCaption){
        g_string_free(theItem->option_gstrCaption, TRUE);
    }
    if (theItem->option_gstrLabel){
        g_string_free(theItem->option_gstrLabel, TRUE);
    }
    if (theItem->DialogEntityList){
        DialogEntityList_freeList ( &(theItem->DialogEntityList));
    }
    if (theItem->gstrWorkingDir){
        g_string_free(theItem->gstrWorkingDir, TRUE);
    }


    free(theItem);

    TRACEIT (7, "UserCommandList_deleteItem              end");
    return bufList;
}


int UserCommandList_freeList(struct _UserCommandList **theList){
    TRACEIT (7, "UserCommandList_freeList              start");
	//has to free _DialogEntityList and glistTargets too.
	//return: -1=error, 0=ok
	if (theList == NULL){
		TRACEIT (2, "UserCommandList_freeList			end, parameter NULL");
		return -1;
	}

	struct _UserCommandList *firstItem=UserCommandList_getFirstItem(*theList);
	struct _UserCommandList *bufItem=firstItem;

	if (firstItem == NULL){
		TRACEIT (2, "UserCommandList_freeList			end, could not get first item in list");
		return -1;
	}

	while (bufItem){
		if (bufItem->gstrName){
			g_string_free(bufItem->gstrName, TRUE);
		}
		if (bufItem->glistTargets){
			g_list_foreach(bufItem->glistTargets, (GFunc) GListFreeGStrings, NULL);
			g_list_free(bufItem->glistTargets);
		}
		if (bufItem->gstrCommand){
			g_string_free(bufItem->gstrCommand, TRUE);
		}
		if (bufItem->gstrDialogCaption){
			g_string_free(bufItem->gstrDialogCaption, TRUE);
		}
		if (bufItem->gstrDialogLabel){
			g_string_free(bufItem->gstrDialogLabel, TRUE);
		}
		if (bufItem->option_gstrQuitreply){
			g_string_free(bufItem->option_gstrQuitreply, TRUE);
		}
		if (bufItem->option_gstrActionnote){
			g_string_free(bufItem->option_gstrActionnote, TRUE);
		}
		if (bufItem->option_gstrCaption){
			g_string_free(bufItem->option_gstrCaption, TRUE);
		}
		if (bufItem->option_gstrLabel){
			g_string_free(bufItem->option_gstrLabel, TRUE);
		}
		if (bufItem->DialogEntityList){
			DialogEntityList_freeList ( &(bufItem->DialogEntityList));
		}
		if (bufItem->gstrWorkingDir){
            g_string_free(bufItem->gstrWorkingDir, TRUE);
        }

		firstItem =  bufItem->nextItem;
		free(bufItem);
		bufItem = firstItem;
	}
	*theList=NULL;

    TRACEIT (7, "UserCommandList_freeList              end");
	return 0;
}


int UserCommandList_editItem(struct _UserCommandList *theItem, int iIsDefaultCommand, const char *cpName, const char *cpTarget, const char *cpCommand, int iCallType, const char *cpOptionQuitreply, const char *cpOptionActionnote, const char *cpOptionCaption, const char *cpOptionLabel, const char *cpDialogCaption, const char *cpDialogLabel, const char *cpWorkingDir){
    TRACEIT (7, "UserCommandList_editItem              start");
	//return: -1=error, 0=ok
	//the target amy be added to the GList
	if (theItem == NULL){
		TRACEIT (2, "UserCommandList_editItem			parameter NULL");
		return -1;
	}

	if (cpOptionActionnote){
		if (theItem->option_gstrActionnote){
			g_string_assign(theItem->option_gstrActionnote, cpOptionActionnote);
		}
		else{
			theItem->option_gstrActionnote = g_string_new(cpOptionActionnote);
		}
	}
	if (cpOptionCaption){
		if (theItem->option_gstrCaption){
			g_string_assign(theItem->option_gstrCaption, cpOptionCaption);
		}
		else{
			theItem->option_gstrCaption = g_string_new(cpOptionCaption);
		}
	}
	if (cpOptionLabel){
		if (theItem->option_gstrLabel){
			g_string_assign(theItem->option_gstrLabel, cpOptionLabel);
		}
		else{
			theItem->option_gstrLabel = g_string_new(cpOptionLabel);
		}
	}
	if (cpDialogCaption){
		if (theItem->gstrDialogCaption){
			g_string_assign(theItem->gstrDialogCaption, cpDialogCaption);
		}
		else{
			theItem->gstrDialogCaption = g_string_new(cpDialogCaption);
		}
	}
	if (cpDialogLabel){
		if (theItem->gstrDialogLabel){
			g_string_assign(theItem->gstrDialogLabel, cpDialogLabel);
		}
		else{
			theItem->gstrDialogLabel = g_string_new(cpDialogLabel);
		}
	}
	if (iIsDefaultCommand != -1){
		theItem->iIsDefaultCommand = iIsDefaultCommand;
	}
	if (cpName){
		if (theItem->gstrName){
			g_string_assign(theItem->gstrName, cpName);
		}
		else{
			theItem->gstrName = g_string_new(cpName);
		}
	}
	if (cpTarget){
		theItem->glistTargets = g_list_append (theItem->glistTargets, g_string_new(cpTarget));
	}
	if (cpCommand){
		if (theItem->gstrCommand){
			g_string_assign(theItem->gstrCommand, cpCommand);
		}
		else{
			theItem->gstrCommand = g_string_new(cpCommand);
		}
	}
	if (iCallType != -1){
		theItem->option_iCallType = iCallType;
	}
	if (cpOptionQuitreply){
		if (theItem->option_gstrQuitreply){
			g_string_assign(theItem->option_gstrQuitreply, cpOptionQuitreply);
		}
		else{
			theItem->option_gstrQuitreply = g_string_new(cpOptionQuitreply);
		}
	}
	if (cpWorkingDir){
		if (theItem->gstrWorkingDir){
			g_string_assign(theItem->gstrWorkingDir, cpWorkingDir);
		}
		else{
			theItem->gstrWorkingDir = g_string_new(cpWorkingDir);
		}
	}

    TRACEIT (7, "UserCommandList_editItem              end");
	return 0;
}


int UserCommandList_deleteItemElement(struct _UserCommandList *theItem, int iName, int iTarget, int iCommand, int iOptionQuitreply, int iOptionActionnote, int iOptionCaption, int iOptionLabel, int iDialogCaption, int iDialogLabel, int iWorkingDir){
    //deletes the GStrings fo the elements specified in the parameters
    //0=do nothing, 1=delete this element
    TRACEIT (7, "UserCommandList_deleteItemElement              start");
	//return: -1=error, 0=ok
	//the target amy be added to the GList
	if (theItem == NULL){
		TRACEIT (2, "UserCommandList_deleteItemElement			parameter NULL");
		return -1;
	}

	if (1==iOptionActionnote){
		if (theItem->option_gstrActionnote){
			g_string_free(theItem->option_gstrActionnote, TRUE);
			theItem->option_gstrActionnote = NULL;
		}
	}
	if (1==iOptionCaption){
		if (theItem->option_gstrCaption){
			g_string_free(theItem->option_gstrCaption, TRUE);
			theItem->option_gstrCaption = NULL;
		}
	}
	if (1==iOptionLabel){
		if (theItem->option_gstrLabel){
			g_string_free(theItem->option_gstrLabel, TRUE);
			theItem->option_gstrLabel = NULL;
		}
	}
	if (1==iDialogCaption){
		if (theItem->gstrDialogCaption){
			g_string_free(theItem->gstrDialogCaption, TRUE);
			theItem->gstrDialogCaption = NULL;
		}
	}
	if (1==iDialogLabel){
		if (theItem->gstrDialogLabel){
			g_string_free(theItem->gstrDialogLabel, TRUE);
			theItem->gstrDialogLabel = NULL;
		}
	}
	if (1==iName){
		if (theItem->gstrName){
			g_string_free(theItem->gstrName, TRUE);
			theItem->gstrName = NULL;
		}
	}
	if (1==iTarget){
	    UserCommandList_clearTargets (theItem);
	}
	if (1==iCommand){
		if (theItem->gstrCommand){
			g_string_free(theItem->gstrCommand, TRUE);
			theItem->gstrCommand = NULL;
		}
	}
	if (1==iOptionQuitreply){
		if (theItem->option_gstrQuitreply){
			g_string_free(theItem->option_gstrQuitreply, TRUE);
			theItem->option_gstrQuitreply = NULL;
		}
	}
	if (1==iWorkingDir){
		if (theItem->gstrWorkingDir){
			g_string_free(theItem->gstrWorkingDir, TRUE);
			theItem->gstrWorkingDir = NULL;
		}
	}

    TRACEIT (7, "UserCommandList_deleteItemElement              end");
	return 0;
}


int UserCommandList_addItemAsCopy(struct _UserCommandList **theList, struct _UserCommandList *theNewItem){
    TRACEIT (7, "UserCommandList_addItemAsCopy              start");
	//simply adds theNewItem to theList, this will preserve us to copy the whole content of hte list (which holds 2 lists by its own!)
	//you can give a second list to 'theNewItem', then, the two lists merge.
	//return: -1=error, 0=ok
	if (theList == NULL ||  theNewItem==NULL){
		TRACEIT (2, "UserCommandList_addItemAsCopy			end, parameter NULL");
		return -1;
	}

    if (*theList==NULL){    //Brand new list, in this case we start one
        *theList = theNewItem;
        TRACEIT (7, "UserCommandList_addItemAsCopy              end, started new list");
        return 0;
    }

	struct _UserCommandList *bufItem = NULL;

	bufItem = UserCommandList_getLastItem (*theList);
	if (bufItem == NULL){
		TRACEIT (2, "UserCommandList_addItemAsCopy			error, I cant find the last item from first list");
		return -1;
	}

	theNewItem = UserCommandList_getFirstItem(theNewItem);
	if (theNewItem == NULL){
		TRACEIT (2, "UserCommandList_addItemAsCopy			error, I cant find the last item from second list");
		return -1;
	}

	bufItem->nextItem = theNewItem;
	theNewItem->prevItem = bufItem;
	//theNewItem->nextItem = NULL;	//do not do this, as we may want to add a List with more than one item

    TRACEIT (7, "UserCommandList_addItemAsCopy              end");
	return 0;
}


struct _UserCommandList *UserCommandList_getLastItem (struct _UserCommandList *theList){
    TRACEIT (7, "UserCommandList_getLastItem              start");
	//return: NULL:error
	if (theList == NULL){
		TRACEIT (2, "UserCommandList_getLastItem			end, parameter NULL");
		return NULL;
	}

	while (theList->nextItem){
		theList = theList->nextItem;
	}

    TRACEIT (7, "UserCommandList_getLastItem              end");
	return theList;
}


struct _UserCommandList *UserCommandList_getFirstItem (struct _UserCommandList *theList){
    TRACEIT (7, "UserCommandList_getFirstItem              start");
	//return: NULL:error
	if (theList == NULL){
		TRACEIT (2, "UserCommandList_getFirstItem			end, parameter NULL");
		return NULL;
	}

	while (theList->prevItem){
		theList = theList->prevItem;
	}

    TRACEIT (7, "UserCommandList_getFirstItem              end");
	return theList;
}


int UserCommandList_copyItemToList (struct _UserCommandList **theList, struct _UserCommandList *NewItem){
    //real copy
	//return: -1=error, 0=ok
	//gets a Listpointer and a allready existing item (in another list) and copies (this means realy copy, allocating new memory)
	//the content of the newItem to the end of theList.
	//only one item is copied from NewItem, no matter if this is a list for its own.

	if (theList == NULL || NewItem == NULL){
        TRACEIT (2, "UserCommandList_copyItemToList             end, parameter NULL");
        return -1;
	}
    struct _UserCommandList *buflist = NULL;
    GList *glistBuf = NULL;

    UserCommandList_addItem ( theList , "UserCommandList_copyItemToList");

    buflist = UserCommandList_getLastItem(*theList);
    if (buflist == NULL){
        TRACEIT (3, "UserCommandList_copyItemToList             end, error retrieving last item of list");
        return -1;
    }

    buflist->iValid = NewItem->iValid;
    buflist->iIsDefaultCommand = NewItem->iIsDefaultCommand;
    if (NewItem->gstrName){
        buflist->gstrName = g_string_new(NewItem->gstrName->str);
    }
    if (NewItem->gstrCommand){
        buflist->gstrCommand = g_string_new(NewItem->gstrCommand->str);
    }
    if (NewItem->gstrDialogCaption){
        buflist->gstrDialogCaption = g_string_new(NewItem->gstrDialogCaption->str);
    }
    if (NewItem->gstrDialogLabel){
        buflist->gstrDialogLabel = g_string_new(NewItem->gstrDialogLabel->str);
    }
    if (NewItem->option_gstrQuitreply){
        buflist->option_gstrQuitreply = g_string_new(NewItem->option_gstrQuitreply->str);
    }
    if (NewItem->option_gstrActionnote){
        buflist->option_gstrActionnote = g_string_new(NewItem->option_gstrActionnote->str);
    }
    if (NewItem->option_gstrCaption){
        buflist->option_gstrCaption = g_string_new(NewItem->option_gstrCaption->str);
    }
    if (NewItem->option_gstrLabel){
        buflist->option_gstrLabel = g_string_new(NewItem->option_gstrLabel->str);
    }
    if (NewItem->gstrWorkingDir){
        buflist->gstrWorkingDir = g_string_new(NewItem->gstrWorkingDir->str);
    }
    buflist->option_iCallType = NewItem->option_iCallType;

    if (NewItem->glistTargets){
        glistBuf = g_list_first(NewItem->glistTargets);
        while (glistBuf){
            UserCommandList_editItem(buflist, -1, NULL, ((GString *)(glistBuf->data))->str, NULL, -1, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
            glistBuf = glistBuf->next;
        }
    }

    struct _DialogEntityList *DialogEntityList = NewItem->DialogEntityList;
    struct _DialogEntityList *DialogEntityListBuf = NULL;
    while (DialogEntityList){
        DialogEntityList_addItem(&(buflist->DialogEntityList), DialogEntityList->iType);
        DialogEntityList_editLastItem   (buflist->DialogEntityList, -1,
                                        (DialogEntityList->gstrVarIdentifier)?DialogEntityList->gstrVarIdentifier->str:NULL,
                                        (DialogEntityList->gstrLabel)?DialogEntityList->gstrLabel->str:NULL,
                                        DialogEntityList->wdgtLabel,
                                        (DialogEntityList->gstrPredefine)?DialogEntityList->gstrPredefine->str:NULL,
                                        DialogEntityList->wdgtEntryBox,
                                        (DialogEntityList->gstrAdditional1)?DialogEntityList->gstrAdditional1->str:NULL,
                                        (DialogEntityList->gstrAdditional2)?DialogEntityList->gstrAdditional2->str:NULL,
                                        DialogEntityList->wdgtAdditional1);

        DialogEntityListBuf = DialogEntityList_getLastItem (buflist->DialogEntityList);
        if (DialogEntityListBuf == NULL){
            TRACEIT(2, "UserCommandList_copyItemToList          unable to retrieve last item of list");
        }
        else{
            DialogEntityListBuf->iValid = DialogEntityList->iValid;
        }
        DialogEntityList = DialogEntityList->nextItem;
    }
    return 0;
}


gboolean UserCommandList_ItemHasTarget (struct _UserCommandList *theItem, char *cpTarget){
    //returns TRUE if theItem has the target cpTarget
    //otherwise FALSE is returned

    if (theItem == NULL || cpTarget == NULL){
        TRACEIT (2, "UserCommandList_ItemHasTarget          end, Parameter NULL");
        return FALSE;
    }
    gboolean bRet = FALSE;

    char *cpDownString1 = NULL;
    char *cpDownString2 = NULL;

    GList *gListBuf = g_list_first (theItem->glistTargets);
    while (gListBuf){
        cpDownString1 = g_ascii_strdown ( ((GString *)(gListBuf->data))->str, strlen(((GString *)(gListBuf->data))->str));
        cpDownString2 = g_ascii_strdown (cpTarget, strlen(cpTarget));
        if ( cpDownString1!=NULL && cpDownString2!=NULL && 0==strcmp( cpDownString1, cpDownString2)) {
            if (cpDownString1)
                g_free(cpDownString1);
            if (cpDownString2)
                g_free(cpDownString2);
            bRet = TRUE;
            break;
        }
        if (cpDownString1)
            g_free(cpDownString1);
        if (cpDownString2)
            g_free(cpDownString2);
        gListBuf = gListBuf->next;
    }
    return bRet;
}


GList *UserCommandList_getCommandsForTarget (struct _UserCommandList *theWholeList, char *cpTarget){
    TRACEIT (7, "UserCommandList_getCommandsForTarget              start");
	//return: NULL=error
	//searches the listitems in theWholeList for their targets to equal cpTarget and creates a new GList
	//which stores the pointer to the listitems matching.
	//YOU HAVE TO FREE THIS LIST SEPERATLY, but not their userdata, as they are the pointer from the other list, use 'g_list_free(LIST)'

	GList *glistItems = NULL;
	struct _UserCommandList *bufList = NULL;
	GList *gListBuf = NULL;

	if (theWholeList == NULL || cpTarget == NULL){
		TRACEIT(2, "UserCommandList_getCommandsForTarget			end, Parameter NULL");
		return NULL;
	}

	bufList = UserCommandList_getFirstItem(theWholeList);
	if (bufList == NULL){
		TRACEIT(2, "UserCommandList_getCommandsForTarget			end, could not get first item of list");
		return NULL;
	}

	while (bufList){
		gListBuf = g_list_first (bufList->glistTargets);
		while (gListBuf){
			if ( 0==strcmp( ((GString *)(gListBuf->data))->str, cpTarget)) {
				//UserCommandList_copyItemToList (&NewList, bufList);
				glistItems = g_list_append (glistItems, (void *)bufList);
			}
			gListBuf = gListBuf->next;
		}
		bufList = bufList->nextItem;
	}
    TRACEIT (7, "UserCommandList_getCommandsForTarget              end");
	return glistItems;
}


int DialogEntityList_addItem(struct _DialogEntityList **theList, int iType){
    TRACEIT (7, "DialogEntityList_addItem              start");
	//if iType == -1, its unknown
	//return: -1=error, 0=ok
	if (theList == NULL){
		TRACEIT (2, "DialogEntityList_addItem			end, parameter NULL");
		return -1;
	}

	struct _DialogEntityList *newItem = malloc (sizeof (struct _DialogEntityList));
	struct _DialogEntityList *bufItem = NULL;
	if (newItem == NULL){
		TRACEIT (2, "DialogEntityList_addItem			Could not allocate memory");
		return -1;
	}

	//default values for member
	newItem->iType = iType;
	newItem->iValid = 0;
	newItem->gstrVarIdentifier = NULL;
	newItem->gstrVarValue = NULL;
	newItem->gstrLabel = NULL;
	newItem->wdgtLabel = NULL;
	newItem->gstrPredefine = NULL;
	newItem->wdgtEntryBox = NULL;
	newItem->gstrAdditional1 = NULL;
	newItem->gstrAdditional2 = NULL;
	newItem->wdgtAdditional1 = NULL;
	newItem->wdgtAdditional2 = NULL;

	newItem->nextItem = NULL;
	newItem->prevItem = NULL;

	//store the new item
	if (*theList == NULL){	//brand new list
		*theList = newItem;
	}
	else{
		bufItem = DialogEntityList_getLastItem (*theList);
		if (bufItem == NULL){
			TRACEIT (2, "DialogEntityList_addItem			error, its no new list, but I also cant find the last item");
			free(newItem);
			return -1;
		}

		bufItem->nextItem = newItem;
		newItem->prevItem = bufItem;
	}

    TRACEIT (7, "DialogEntityList_addItem              end");
	return 0;
}


int DialogEntityList_freeList(struct _DialogEntityList **theList){
    TRACEIT (7, "DialogEntityList_freeList              start");
	if (theList == NULL){
		TRACEIT (2, "DialogEntityList_freeList			end, parameter NULL");
		return -1;
	}

	struct _DialogEntityList *firstItem=DialogEntityList_getFirstItem(*theList);
	struct _DialogEntityList *bufItem=firstItem;

	if (firstItem == NULL){
		TRACEIT (2, "DialogEntityList_freeList			end, could not get first item in list");
		return -1;
	}

	while (bufItem){
		if (bufItem->gstrVarIdentifier){
			g_string_free(bufItem->gstrVarIdentifier, TRUE);
		}
		if (bufItem->gstrVarValue){
            g_string_free(bufItem->gstrVarValue, TRUE);
		}
		if (bufItem->gstrLabel){
			g_string_free(bufItem->gstrLabel, TRUE);
		}
		if (bufItem->gstrPredefine){
			g_string_free(bufItem->gstrPredefine, TRUE);
		}
		if (bufItem->gstrAdditional1){
			g_string_free(bufItem->gstrAdditional1, TRUE);
		}
		if (bufItem->gstrAdditional2){
			g_string_free(bufItem->gstrAdditional2, TRUE);
		}
		if (bufItem->wdgtLabel){
			g_object_unref(bufItem->wdgtLabel);
		}
		if (bufItem->wdgtEntryBox){
			g_object_unref(bufItem->wdgtEntryBox);
		}
		if (bufItem->wdgtAdditional1){
			g_object_unref(bufItem->wdgtAdditional1);
		}
		if (bufItem->wdgtAdditional2){
		    g_object_unref(bufItem->wdgtAdditional2);
		}

		firstItem =  bufItem->nextItem;
		free(bufItem);
		bufItem = firstItem;
	}
	*theList = NULL;

    TRACEIT (7, "DialogEntityList_freeList              end");
	return 0;
}


int DialogEntityList_editItem(struct _DialogEntityList *theItem, int iType, char *cpVarID, char *cpLabel, GtkWidget *wdgtLabel, char *cpPredefine, GtkWidget *wdgtEntryBox, char *cpAdd1, char *cpAdd2, GtkWidget *wdgtAdd1 ){
    TRACEIT (7, "DialogEntityList_editItem              start");
	//return: -1=error, 0=ok
	//changes the content of the given item

	if (theItem == NULL){
		TRACEIT(2, "DialogEntityList_editItem				end, parameter NULL");
		return -1;
	}


	if (iType != -1){
		theItem->iType = iType;
	}
	if (cpVarID){
		if (theItem->gstrVarIdentifier){
			g_string_assign(theItem->gstrVarIdentifier, cpVarID);
		}
		else{
			theItem->gstrVarIdentifier = g_string_new(cpVarID);
		}
	}
	if (cpLabel){
		if (theItem->gstrLabel){
			g_string_assign(theItem->gstrLabel, cpLabel);
		}
		else{
			theItem->gstrLabel = g_string_new(cpLabel);
		}
	}
	if (cpPredefine){
		if (theItem->gstrPredefine){
			g_string_assign(theItem->gstrPredefine, cpPredefine);
		}
		else{
			theItem->gstrPredefine = g_string_new(cpPredefine);
		}
	}
	if (cpAdd1){
		if (theItem->gstrAdditional1){
			g_string_assign(theItem->gstrAdditional1, cpAdd1);
		}
		else{
			theItem->gstrAdditional1 = g_string_new(cpAdd1);
		}
	}
	if (cpAdd2){
		if (theItem->gstrAdditional2){
			g_string_assign(theItem->gstrAdditional2, cpAdd2);
		}
		else{
			theItem->gstrAdditional2 = g_string_new(cpAdd2);
		}
	}
	if (wdgtLabel){
		if (theItem->wdgtLabel){
			g_object_unref(theItem->wdgtLabel);
		}
		theItem->wdgtLabel = wdgtLabel;
	}
	if (wdgtEntryBox){
		if (theItem->wdgtEntryBox){
			g_object_unref(theItem->wdgtEntryBox);
		}
		theItem->wdgtEntryBox = wdgtEntryBox;
	}
	if (wdgtAdd1){
		if (theItem->wdgtAdditional1){
			g_object_unref(theItem->wdgtAdditional1);
		}
		theItem->wdgtAdditional1 = wdgtAdd1;
	}

    TRACEIT (7, "DialogEntityList_editItem              end");
	return 0;
}


int DialogEntityList_editLastItem(struct _DialogEntityList *theList, int iType, char *cpVarID, char *cpLabel, GtkWidget *wdgtLabel, char *cpPredefine, GtkWidget *wdggtEntryBox, char *cpAdd1, char *cpAdd2, GtkWidget *wdgtAdd1 ){
    TRACEIT (7, "DialogEntityList_editLastItem              start");
	//return: -1=error, 0=ok
	//searches the last item in the list and changes its content

	if (theList == NULL){
		TRACEIT(2, "DialogEntityList_editItem				end, parameter NULL");
		return -1;
	}
	struct _DialogEntityList *bufItem = DialogEntityList_getLastItem(theList);

	DialogEntityList_editItem(bufItem, iType, cpVarID, cpLabel, wdgtLabel, cpPredefine, wdggtEntryBox, cpAdd1, cpAdd2, wdgtAdd1);

    TRACEIT (7, "DialogEntityList_editLastItem              end");
    return 0;

}


struct _DialogEntityList *DialogEntityList_getLastItem(struct _DialogEntityList *theList){
    TRACEIT (7, "DialogEntityList_getLastItem              start");
	//return: NULL:error
	if (theList == NULL){
		TRACEIT (2, "DialogEntityList_getLastItem			end, parameter NULL");
		return NULL;
	}

	while (theList->nextItem){
		theList = theList->nextItem;
	}

    TRACEIT (7, "DialogEntityList_getLastItem              end");
	return theList;
}


struct _DialogEntityList *DialogEntityList_getFirstItem(struct _DialogEntityList *theList){
    TRACEIT (7, "DialogEntityList_getFirstItem              start");
	//return: NULL:error
	if (theList == NULL){
		TRACEIT (2, "DialogEntityList_getLastItem			end, parameter NULL");
		return NULL;
	}

	while (theList->prevItem){
		theList = theList->prevItem;
	}

    TRACEIT (7, "DialogEntityList_getFirstItem              end");
	return theList;
}


int XMLRelation_updateByElement(struct _Items_XML_Relation *relation, const char *cpElement, gboolean bStartEnd, gboolean bMakeItThreadSave){
    TRACEIT (7, "XMLRelation_updateByElement              start");
	//updates the relation bools in respect to cpElement and bStartEnd
	//if bStartEnd is true, its starting, new value of relation bool is TRUE, otherwise its FALSE
	//return: -1=error, 0=notfound, 1=found/updated, ////// NO::::2=unknown state, but bools are valid again
	if (relation == NULL || cpElement == NULL){
		TRACEIT (2, "XMLRelation_updateByElement				end, Parameter NULL");
		return -1;
	}
	int ret = 1;

	struct _OptionsList	*KeyWords = NULL;

    //printf("XMLRelation_updateByElement: relationPointer: %p", relation);
    //when we are in end-mode, the keyword may not be known. (Indeed, our function is not made for end-mode operation but only for start-mode)
    //end mode operation is much work, as any of the keywords can end without closing one of the items opened in between.
    //so we skipp this tests in case we are in end-mode
    if (bStartEnd == TRUE){
        KeyWords = XMLParser_getValidKeyWordsForState(relation);
        if (KeyWords == NULL){
            TRACEIT (4, "XMLRelation_updateByElement           end, context error:");
            TRACEIT (4, cpElement);
            /*
            if (relation == NULL){
                printf("WHY DIDNT I GOT AN ERROR?\n");
            }else{
               printf("relation = %d\n",relation->iRelation);
            }*/
            return 0;
        }
        if ( NULL == OptionsList_getItemByName (KeyWords, cpElement) ){
            TRACEIT (4, "XMLRelation_updateByElement           end, element name not known in this context:");
            TRACEIT (4, cpElement);
            OptionsList_freeList(&KeyWords);
            return 0;
        }
    }

    gboolean *theBoolToChange = NULL;

    //we do not need to check all circumstances as we allready made sure the new element name is available in the current relation.
    //all we have to do is to change the value of the right bool:

    if (0 == strcmp ("CatsEyE", cpElement)){
    	theBoolToChange = &(relation->bCatsEyEBlock);
    }
    else if (0 == strcmp ("CatsEyEBookmarks", cpElement)){
    	theBoolToChange = &(relation->bCatsEyEBookMarksBlock);
    }
    else if (0 == strcmp ("CatsEyEOptions", cpElement)){
    	theBoolToChange = &(relation->bCatsEyEOptionsBlock);
    }
    else if (0 == strcmp ("CatsEyEUsercommands", cpElement)){
    	theBoolToChange = &(relation->bCatsEyEUserCommandsBlock);
	}
    else if (0 == strcmp ("Option", cpElement)){
    	theBoolToChange = &(relation->bOptionBlock);
    }
    else if (0 == strcmp ("ID", cpElement)){
    	theBoolToChange = &(relation->bIDBlock);
    }
    else if (0 == strcmp ("Name", cpElement)){
    	theBoolToChange = &(relation->bNameBlock);
    }
    else if (0 == strcmp ("Value", cpElement)){
    	theBoolToChange = &(relation->bValueBlock);
    }
    else if (0 == strcmp ("Usercommand", cpElement)){
    	theBoolToChange = &(relation->bUserCommandBlock);
    }
    else if (0 == strcmp ("Object", cpElement)){
    	theBoolToChange = &(relation->bObjectBlock);
    }

    if (theBoolToChange != NULL){
        if (*theBoolToChange == bStartEnd){
            TRACEIT (4, "XMLRelation_updateByNewElement           Trying to enter relation we are allready in (enter or leave):");
            TRACEIT (4, cpElement);
        }
        *theBoolToChange = bStartEnd;
    }

    XMLRelation_accumulateRelations(relation);

    //now we check if the current state is valid
    //int iValid = 0;

    if (    ( 0 ) == relation->iRelation ||
    		( 2 ) == relation->iRelation ||
            ( 2 + 4 ) == relation->iRelation ||
            ( 2 + 4 + 128 ) == relation->iRelation ||
            ( 2 + 4 + 128 + 256 ) == relation->iRelation ||
            ( 2 + 4 + 128 + 512 ) == relation->iRelation ||
            ( 2 + 4 + 128 + 1024 ) == relation->iRelation ||
            ( 2 + 8 ) == relation->iRelation ||
            ( 2 + 8 + 128 ) == relation->iRelation ||
            ( 2 + 8 + 128 + 256 ) == relation->iRelation ||
            ( 2 + 8 + 128 + 512 ) == relation->iRelation ||
            ( 2 + 8 + 128 + 1024 ) == relation->iRelation ||
            ( 2 + 16 ) == relation->iRelation ||
            ( 2 + 16 + 32 ) == relation->iRelation ||
            ( 2 + 16 + 32 + 64) == relation->iRelation ||
            ( 2 + 16 + 32 + 64 + 128 ) == relation->iRelation ||
            ( 2 + 16 + 32 + 64 + 128 + 256 ) == relation->iRelation ||
            ( 2 + 16 + 32 + 64 + 128 + 512 ) == relation->iRelation ||
            ( 2 + 16 + 32 + 64 + 128 + 1024 ) == relation->iRelation ||
            ( 2 + 16 + 32 + 128) == relation->iRelation ||
            ( 2 + 16 + 32 + 128 + 256 ) == relation->iRelation ||
            ( 2 + 16 + 32 + 128 + 512 ) == relation->iRelation ||
            ( 2 + 16 + 32 + 128 + 1024 ) == relation->iRelation
        ){
            relation->bUnKnownBlock = FALSE;
            ret=1;	//everything is fine
    }
    else{
        relation->bUnKnownBlock = TRUE;
        ret=2;	//errorcode for block error
    }

/*
	//now, if the relation is not valid, we make it valid.
	//e.g. when we expect closing some option block but the file closed the command block or even the catseye block
	//before, we have to close the optionsblock - bool and all others too.
	if (relation->bUnKnownBlock == FALSE){

	}
*/

    XMLRelation_accumulateRelations(relation);
    if (KeyWords){
        OptionsList_freeList(&KeyWords);
    }

    TRACEIT (7, "XMLRelation_updateByElement              end");
    return 1;
}


void XMLParser_foundStart (GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error){
    TRACEIT (7, "XMLParser_foundStart              start");
    struct _Parser_Mode_Struct *info = (struct _Parser_Mode_Struct *)data;
    if (info == NULL){
        TRACEIT (2, "XMLParser_foundStart           end, Parameter NULL");
        return;
    }

    struct _Items_XML_Relation *relation = (struct _Items_XML_Relation *)info->relation;
    if (info->relation==NULL || relation==NULL){
        TRACEIT (2, "XMLParser_foundStart           end, Parameter 2 NULL");
        return;
    }

    //is the element name valid in our current relation?
	if (1!=XMLRelation_updateByElement (relation, element_name, TRUE, info->bMakeItThreadSave)){
		TRACEIT (4, "XMLParser_foundStart           end, error while parsing configfile");
		return;
	}
    //printf("foundstart: name: %s\n",element_name);
	//store the attributes to the info structm this is used in UserCommand part, where the <Object> Block has a type attribute to identify the object e.g. as question
	info->cppAttributeValues = attribute_values;
    info->cppAttributeNames = attribute_names;

	//the rest is done within this notification functions
	//but we have to set them first in order to the mode we are in
	if (info->functionStartNotification != NULL){
		info->functionStartNotification = NULL;
	}

	if ( relation->iRelation & (4) ){	//bookmarks block
		info->functionStartNotification = Options_FoundStartCallback;
		info->iCurrentListType = CONFIGFILE_LOAD_BOOKMARKS;
		info->storageList = (void *)&g_BookMarkList;
	}
	else if ( relation->iRelation & (8) ){	//options block
		info->functionStartNotification = Options_FoundStartCallback;
		info->iCurrentListType = CONFIGFILE_LOAD_OPTIONS;
		info->storageList = (void *)&g_OptionsList;
	}
	else if ( relation->iRelation & (16) ){	//usercommands block
		info->functionStartNotification = Usercommand_FoundStartCallback;
		info->iCurrentListType = CONFIGFILE_LOAD_USERCOMMANDS;
		info->storageList = (void *)&g_UserCommandList;
	}

	if (NULL != info->functionStartNotification){
		info->functionStartNotification( info );
	}

    //MyMessageWidget(NULL,"XMLParser_foundStart",(char *)element_name);

    TRACEIT (7, "XMLParser_foundStart              end");
    return;
}


void XMLParser_foundEnd (GMarkupParseContext *context, const gchar *element_name, gpointer data, GError **error){
    TRACEIT (7, "XMLParser_foundEnd              start");
    //struct AllInformationAllUseStruct *info = (struct AllInformationAllUseStruct *)data;
    struct _Parser_Mode_Struct *info = (struct _Parser_Mode_Struct *)data;
    if (info == NULL){
        TRACEIT (2, "XMLParser_foundEnd           end, Parameter NULL");
        return;
    }
    struct _Items_XML_Relation *relation = info->relation;
    if ( info->relation==NULL){
        TRACEIT (2, "XMLParser_foundEnd           end, Parameter 2 NULL");
        return;
    }

    //now we can update the relation
	if (1!=XMLRelation_updateByElement (relation, element_name, FALSE, info->bMakeItThreadSave)){
		TRACEIT (4, "XMLParser_foundEnd           end, error while parsing configfile");
		return;
	}
    //printf("foundEnd: name: %s\n",element_name);
	//Now the end-function can handle the new state
	//but we have to set them first in order to the mode we are in
	if (info->functionStopNotification != NULL){
		info->functionStopNotification = NULL;
	}

	if ( relation->iRelation & (4) ){	//bookmarks block
		info->functionStopNotification = Options_FoundEndCallback;
	}
	else if ( relation->iRelation & (8) ){	//options block
		info->functionStopNotification = Options_FoundEndCallback;
	}
	else if ( relation->iRelation & (16) ){	//usercommands block
		info->functionStopNotification = Usercommand_FoundEndCallback;
	}
	else{
		info->iCurrentListType = CONFIGFILE_LOAD_NOTHING;
		freeRelicts(info);
		info->storageList = NULL;
	}

	if (NULL != info->functionStopNotification){
		info->functionStopNotification( info, element_name );
	}

    //MyMessageWidget(NULL,"XMLParser_foundEnd",(char *)element_name);

    TRACEIT (7, "XMLParser_foundEnd              end");
    return;
}


void XMLParser_foundText (GMarkupParseContext *context, const gchar *text, gsize text_len, gpointer data, GError **error){
    TRACEIT (7, "XMLParser_foundText              start");
    if (text_len < 1){
    	TRACEIT (6, "XMLParser_foundText           end, no text found, why we are here?");
        return;
    }

    struct _Parser_Mode_Struct *info = (struct _Parser_Mode_Struct *)data;
    if (info == NULL){
        TRACEIT (2, "XMLParser_foundText           end, Parameter NULL");
        return;
    }

    struct _Items_XML_Relation *relation = (struct _Items_XML_Relation *)info->relation;
    if ( info->relation==NULL){
        TRACEIT (2, "XMLParser_foundText           end, Parameter 2 NULL");
        return;
    }

    char *buf = malloc(sizeof(char) * (text_len+2));
    memcpy(buf, text, text_len);
    buf[text_len]='\0';
    //printf("foundText: text: %s\n",buf);
    //char buf2[1500];
    //sprintf(buf2, "textlen=%d\ntext=%s",text_len,buf);

    //MyMessageWidget(NULL,"foundtext",buf2);

	//Now the text-function can handle the new state
	//but we have to set them first in order to the mode we are in
	if (info->functionTextNotification != NULL){
		info->functionTextNotification = NULL;
	}

	if ( relation->iRelation & (4) ){	//bookmarks block
		info->functionTextNotification = Options_FoundTextCallback;
	}
	else if ( relation->iRelation & (8) ){	//options block
		info->functionTextNotification = Options_FoundTextCallback;
	}
	else if ( relation->iRelation & (16) ){	//usercommands block
		info->functionTextNotification = Options_FoundTextCallback;
	}

	if (NULL != info->functionTextNotification){
		info->functionTextNotification( info, buf );
	}

    free(buf);

    TRACEIT (7, "XMLParser_foundText              end");
}


int XMLParser_ParseFileFile (struct myFiles_struct *myFilestru, struct _Parser_Mode_Struct *modeStruct, gboolean bMakeItThreadSave){
    TRACEIT (7, "XMLParser_ParseFileFile              start");
	//myFiles_struct has to be an opened filepointer.
    //the file has to be xml style, we do not manipulate the filepointer before sending lines to the parser.
    //this means we DO NOT rewind the filepointer.
    //return: 0=ok, -1=error
    if (myFilestru==NULL || myFilestru->File == NULL){
		TRACEIT (2, "XMLParser_ParseFileFile			end, PARAMETER NULL");
		return -1;
	}
	if (modeStruct==NULL || modeStruct->relation==NULL){
		TRACEIT (2, "XMLParser_ParseFileFile			end, PARAMETER  2 NULL");
		return -1;
	}

    GString *gstrBuf=g_string_sized_new(10);

    //fill the parser struct.
    GMarkupParser Parser;
    Parser.start_element = XMLParser_foundStart;
    Parser.end_element = XMLParser_foundEnd;
    Parser.text         = XMLParser_foundText;
    Parser.passthrough = NULL;
    Parser.error = NULL;
    GError *err=NULL;

    //create parser
    GMarkupParseContext *cont = g_markup_parse_context_new (&Parser, 0, (void *)modeStruct, NULL); //G_MARKUP_TREAT_CDATA_AS_TEXT

    //lets start parsing
    char line[5001];
    while ( NULL != fgets(line, 5000, myFilestru->File)){
        if (FALSE == g_markup_parse_context_parse (cont, line, strlen(line), &err)){
            g_string_printf(gstrBuf, "Error parsing configfile:\n %s\n\nError:\n%d\t%s", myFilestru->gstrFileName->str, err->code, err->message);
            if (bMakeItThreadSave == TRUE)
                myWrap_gdk_threads_enter("XMLParser_ParseFileFile 2");

            MyMessageWidget(NULL, "Parser Error", gstrBuf->str);

            if (bMakeItThreadSave == TRUE)
                myWrap_gdk_threads_leave("XMLParser_ParseFileFile 2");

            TRACEIT (5, "XMLParser_ParseFileFile            Error parsing configfile:");
            TRACEIT (5, gstrBuf->str);
            break;
        }
        //debugOut_PrintLists();
    }

    g_markup_parse_context_free (cont);
	g_string_free(gstrBuf, TRUE);

    TRACEIT (7, "XMLParser_ParseFileFile              end");
	return 0;
}


int Options_CreateDefaultList (){
    TRACEIT (7, "Options_CreateDefaultList              start");

    GString *gstrBuf = g_string_sized_new(20);
    getHomeDirectory(gstrBuf);

    if (g_OptionsList != NULL){
        OptionsList_freeList (&g_OptionsList);
    }

    OptionsList_addItem(&g_OptionsList, STANDARDSTARTDIRECTORY, NULL, gstrBuf->str);
    OptionsList_addItem(&g_OptionsList, COLOR_ITEMS_DEFAULT, NULL, DEFAULT_COLOR_ITEMS_DEFAULT);
    OptionsList_addItem(&g_OptionsList, COLOR_DIRECTORIES, NULL, DEFAULT_COLOR_DIRECTORIES);
    OptionsList_addItem(&g_OptionsList, COLOR_LINKS, NULL, DEFAULT_COLOR_LINKS);
    OptionsList_addItem(&g_OptionsList, COLOR_ITEMS_MISSING, NULL, DEFAULT_COLOR_ITEMS_MISSING);

    g_string_free(gstrBuf, TRUE);

    TRACEIT (7, "Options_CreateDefaultList              end");
    return 0;
}


struct _OptionsList *OptionsList_getItemByName(struct _OptionsList *OptionsList, const char *cpName){
    TRACEIT (7, "OptionsList_getItemByName              start");
    //return NULL=error, not found,     or pointer to listitem
    if (OptionsList == NULL || cpName == NULL){
        TRACEIT (2, "OptionsList_getItemByName              end, Parameter NULL");
        return NULL;
    }

    struct _OptionsList *bufList = OptionsList_getFirstItem(OptionsList);

    while (bufList){
        if (bufList->gstrName){
            if (0 == strcmp(bufList->gstrName->str, cpName)){
                return bufList;
            }
        }
        bufList = bufList->nextItem;
    }

    TRACEIT (7, "OptionsList_getItemByName              end");
    return NULL;
}


struct _OptionsList *OptionsList_getItemByID(struct _OptionsList *OptionsList, const char *cpID){
    TRACEIT (7, "OptionsList_getItemByID              start");
    //return NULL=error, not found,     or pointer to listitem
    if (OptionsList == NULL || cpID == NULL){
        TRACEIT (2, "OptionsList_getItemByID              end, Parameter NULL");
        return NULL;
    }

    struct _OptionsList *bufList = OptionsList_getFirstItem(OptionsList);

    while (bufList){
        if (bufList->gstrID){
            if (0 == strcmp(bufList->gstrID->str, cpID)){
                return bufList;
            }
        }
        bufList = bufList->nextItem;
    }

    TRACEIT (7, "OptionsList_getItemByID              end");
    return NULL;
}


const char *OptionsList_getValueAndNextItemByID (struct _OptionsList *StartItem, struct _OptionsList **NextItem, const char *cpID){
    TRACEIT (7, "OptionsList_getValueAndNextItemByID              start");
    //returns the value of the first item in StartItem list with ID cpID
    //and sets NextItem to the next item which has the same ID
    //example
    //if StartItem has cpID, it returns StartItems value, then searches for the next occurence of ID staring with the item next to StartItem
    //and writes its adress to NextItem
    //so, you can use NextItem as StartItem for future calls to this function and get one value after another
    //if StartItem does not if cpID, it searches for the first occurence of cpID in the list, returns its value, searches for the next occurence
    //and writes this pointer to NextItem
    if (StartItem == NULL || cpID == NULL || NextItem==NULL){
        TRACEIT (2, "OptionsList_getValueAndNextItemByID              end, Parameter NULL");
        return NULL;
    }

    *NextItem = NULL;
    const char *ccpBuf = NULL;
    //search first occurence of cpID
    while (StartItem){
        if (StartItem->gstrID){
            if (0 == strcmp(StartItem->gstrID->str, cpID)){
                if (StartItem->gstrValue){
                     ccpBuf =  (const char *)StartItem->gstrValue->str;
                     break;
                }
            }
        }
        StartItem = StartItem->nextItem;
    }

    //search for the next cpID item
    if (StartItem){
        StartItem = StartItem->nextItem;
        while (StartItem){
            if (StartItem->gstrID){
                if (0 == strcmp(StartItem->gstrID->str, cpID)){
                    *NextItem = StartItem;
                    break;
                }
            }
            StartItem = StartItem->nextItem;
        }
    }

    TRACEIT (7, "OptionsList_getValueAndNextItemByID              end");
    return ccpBuf;
}


const char *OptionsList_getNameByID (struct _OptionsList *OptionsList, const char *cpID){
    TRACEIT (7, "OptionsList_getNameByID              start");
    //return NULL=error, not found,     or pointer to listitem
    if (cpID == NULL){
        TRACEIT (2, "OptionsList_getNameByID              end, Parameter cpID NULL");
        return NULL;
    }
    if (OptionsList == NULL){
        TRACEIT (6, "OptionsList_getNameByID              end, Parameter OptionsList NULL, no configfile found?");
        return NULL;
    }

    struct _OptionsList *bufList = OptionsList_getFirstItem(OptionsList);

    while (bufList){
        if (bufList->gstrID){
            if (0 == strcmp(bufList->gstrID->str, cpID)){
                if (bufList->gstrName){
                    TRACEIT (7, "OptionsList_getNameByID              end");
                    return (const char *)bufList->gstrName->str;
                }
                else{
                    TRACEIT (6, "OptionsList_getNameByID              end, value not set");
                    return NULL;
                }
            }
        }
        bufList = bufList->nextItem;
    }

    TRACEIT (7, "OptionsList_getNameByID              end not found");
    return NULL;
}


int OptionsList_GetIntValueByID (struct _OptionsList *OptionsList, const char *cpID, int iDefault){
	TRACEIT (10, "OptionsList_GetIntValueByID		start");
	if (OptionsList == NULL || cpID == NULL){
		TRACEIT (2, "OptionsList_GetIntValueByID		Parameter NULL");
		return iDefault;
	}

	const char *ccpValue = NULL;
	int iValue = iDefault;
	ccpValue = OptionsList_getValueByID (OptionsList, cpID);
	if (ccpValue != NULL){
		sscanf (ccpValue, "%d", &iValue);
	}

	TRACEIT (10, "OptionsList_GetIntValueByID		end");
	return iValue;
}


const char *OptionsList_getValueByID (struct _OptionsList *OptionsList, const char *cpID){
    TRACEIT (7, "OptionsList_getValueByID              start");
    //return NULL=error, not found,     or pointer to listitem
    if (cpID == NULL){
        TRACEIT (2, "OptionsList_getValueByID              end, Parameter cpID NULL");
        return NULL;
    }
    if (OptionsList == NULL){
        TRACEIT (6, "OptionsList_getValueByID              end, Parameter OptionsList NULL, no configfile found?");
        return NULL;
    }


    struct _OptionsList *bufList = OptionsList_getFirstItem(OptionsList);

    while (bufList){
        if (bufList->gstrID){
            if (0 == strcmp(bufList->gstrID->str, cpID)){
                if (bufList->gstrValue){
                    TRACEIT (7, "OptionsList_getValueByID              end");
                    return (const char *)bufList->gstrValue->str;
                }
                else{
                    TRACEIT (6, "OptionsList_getValueByID              end, value not set");
                    return NULL;
                }
            }
        }
        bufList = bufList->nextItem;
    }

    TRACEIT (7, "OptionsList_getValueByID              end not found");
    return NULL;
}



int OptionsList_editItem(struct _OptionsList *OptionsList, const char*cpID, const char *cpName, const char *cpValue){
    TRACEIT (7, "OptionsList_editItem              start");
    if (OptionsList == NULL){
        TRACEIT(2, "OptionsList_editItem             end, PARAMETER NULL");
        return -1;
    }
    if (cpID != NULL){
        if (OptionsList->gstrID != NULL){
            g_string_assign(OptionsList->gstrID, cpID);
        }
        else{
            OptionsList->gstrID = g_string_new(cpID);
        }
    }
    if (cpName != NULL){
        if (OptionsList->gstrName != NULL){
            g_string_assign(OptionsList->gstrName, cpName);
        }
        else{
            OptionsList->gstrName = g_string_new(cpName);
        }
    }
    if (cpValue != NULL){
        if (OptionsList->gstrValue != NULL){
            g_string_assign(OptionsList->gstrValue, cpValue);
        }
        else{
            OptionsList->gstrValue = g_string_new(cpValue);
        }
    }

    TRACEIT (7, "OptionsList_editItem              end");
    return 0;
}


int OptionsList_addItem(struct _OptionsList **OptionsList, const char*cpID, const char *cpName, const char *cpValue){
    TRACEIT (7, "OptionsList_addItem              start");
    //cpName and/or cpvalue can be NULL, in this case, the GString pointer (either gstrName or gstrValue become NULL)
    //if *OptionsList is NULL, a new list is created.
    //return: -1=error, 0=ok

    if (OptionsList == NULL){
        TRACEIT(2, "OptionsList_addItem             end, PARAMETER NULL");
        return -1;
    }

    struct _OptionsList *newItem = malloc( sizeof(struct _OptionsList) );
    struct _OptionsList *bufList;

    newItem->gstrID = NULL;
    newItem->gstrName = NULL;
    newItem->gstrValue = NULL;
    newItem->nextItem = NULL;
    newItem->prevItem = NULL;

    if (cpID != NULL){
        newItem->gstrID = g_string_new(cpID);
    }
    if (cpName != NULL){
        newItem->gstrName = g_string_new(cpName);
    }
    if (cpValue != NULL){
        newItem->gstrValue = g_string_new(cpValue);
    }

    if (*OptionsList == NULL){
        //brand new list
        *OptionsList = newItem;
    }
    else{
        //new item at end of list
        bufList = OptionsList_getLastItem(*OptionsList);
        if (bufList != NULL){
            bufList->nextItem = newItem;
            newItem->prevItem = bufList;
        }
        else{
            TRACEIT(2, "OptionsList_addItem         weird error");
        }
    }

    TRACEIT (7, "OptionsList_addItem              end");
    return 0;
}


struct _OptionsList *OptionsList_getLastItem(struct _OptionsList *thelist){
    TRACEIT (7, "OptionsList_getLastItem              start");

    if (thelist == NULL){
        TRACEIT (2, "OptionsList_getLastItem        end, PARAMETER NULL");
        return NULL;
    }

    struct _OptionsList *bufList = thelist;

    while (bufList->nextItem){
        bufList = bufList->nextItem;
    }

    TRACEIT (7, "OptionsList_getLastItem              end");
    return bufList;
}


struct _OptionsList *OptionsList_getFirstItem(struct _OptionsList *thelist){
    TRACEIT (7, "OptionsList_getFirstItem              start");
    if (thelist == NULL){
        TRACEIT (2, "OptionsList_getFirstItem        end, PARAMETER NULL");
        return NULL;
    }

    struct _OptionsList *bufList = thelist;

    while (bufList->prevItem){
        bufList = bufList->prevItem;
    }

    TRACEIT (7, "OptionsList_getFirstItem              end");
    return bufList;
}


struct _OptionsList *OptionsList_deleteItem(struct _OptionsList *theItem){
    //deletes theItem and returns the new begining of the list,
    //if theItem was the last item NULL is returned
    TRACEIT (7, "OptionsList_deleteItem              start");
    struct _OptionsList *bufList = NULL;
    if (theItem == NULL){
        TRACEIT (2, "OptionsList_deleteItem        end, PARAMETER NULL");
        return NULL;
    }
    if (theItem->nextItem){
        theItem->nextItem->prevItem = theItem->prevItem;
        bufList = theItem->nextItem;
    }
    if (theItem->prevItem){
        theItem->prevItem->nextItem = theItem->nextItem;
        bufList = theItem->prevItem;
    }

    if (bufList){
        bufList = OptionsList_getFirstItem (bufList);
    }

    if (theItem->gstrID){
        g_string_free(theItem->gstrID, TRUE);
    }
    if (theItem->gstrName){
        g_string_free(theItem->gstrName, TRUE);
    }
    if (theItem->gstrValue){
        g_string_free(theItem->gstrValue, TRUE);
    }
    free(theItem);

    TRACEIT (7, "OptionsList_getFirstItem              end");
    return bufList;
}


void OptionsList_freeList(struct _OptionsList **thelist){
    TRACEIT (7, "OptionsList_freeList              start");
    if (thelist == NULL){
        TRACEIT (2, "OptionsList_freeList        end, PARAMETER NULL");
        return;
    }

    struct _OptionsList *bufList = OptionsList_getFirstItem(*thelist);
    struct _OptionsList *buf2List=NULL;

    while (bufList){
	    if (bufList->gstrID){
            g_string_free(bufList->gstrID, TRUE);
        }
        if (bufList->gstrName){
            g_string_free(bufList->gstrName, TRUE);
        }
        if (bufList->gstrValue){
            g_string_free(bufList->gstrValue, TRUE);
        }
        buf2List=bufList;
        bufList = bufList->nextItem;
        free(buf2List);
    }
    *thelist = NULL;

    TRACEIT (7, "OptionsList_freeList              end");
}


void freeRelicts(struct _Parser_Mode_Struct *info){
    TRACEIT (7, "freeRelicts              start");
	//deletes currentItems in this struct if necessary
	if (info == NULL){
		TRACEIT (2,"freeRelicts				end, parameter NULL");
		return;
	}

	if (info->currentListItem){
	    if (info->iCurrentListType == CONFIGFILE_LOAD_OPTIONS || info->iCurrentListType == CONFIGFILE_LOAD_BOOKMARKS){
            OptionsList_freeList((struct _OptionsList **)&(info->currentListItem));
            //info->currentListItem = NULL;
	    }
	    else if ( info->iCurrentListType == CONFIGFILE_LOAD_USERCOMMANDS){
            UserCommandList_freeList ((struct _UserCommandList **)&(info->currentListItem));
            //info->currentListItem = NULL;
	    }
	    else {
            TRACEIT(3, "freeRelicts             could not determine currentList item to free it But NULL it anyway. THIS DEFINITLY WILL CREATE A MEMORYLEAK! There is something wrong in here.");
            info->currentListItem = NULL;
        }
	}
	if (info->currentOptionBlock){
        OptionsList_freeList((struct _OptionsList **)&(info->currentOptionBlock));
		//info->currentOptionBlock = NULL;
	}
    TRACEIT (7, "freeRelicts              end");
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////NEW FILE HANDLING ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

int myFiles_SaveFileSimpleSave(char *cpLine, char *Mode){
    if (cpLine == NULL || Mode==NULL){
        TRACEIT(2, "myFiles_SaveFileSimpleSave          parameter NULL");
        return -1;
    }
    struct myFiles_struct *stru = myFiles_openSaveFile(Mode);
    if(stru){
        myFiles_SaveValueToFile(stru, cpLine);
        myFiles_closeFile(stru);
        return 0;
    }
    return -1;
}


struct myFiles_struct *myFiles_openSaveFile(char *cpMode){
    TRACEIT(10,"myFiles_saveFile		start");
    if (cpMode==NULL){
        TRACEIT(2, "myFiles_openSaveFile          parameter NULL");
        return NULL;
    }

	GString *SaveFile=g_string_sized_new(20);
    getHomeDirectory (SaveFile);
	checkPathForLastChar(SaveFile);
	g_string_append(SaveFile,SAVE_FILE);

    struct myFiles_struct *stru;
    stru = myFiles_openFile(SaveFile->str, cpMode);

	g_string_free(SaveFile,TRUE);
	TRACEIT(10,"myFiles_saveFile		ends");
    return stru;
}


int myFiles_SaveFileSimpleGetValue(char *Identifier, GString *Value){
	//return 1-> found, return !=0 Not found
    //All spaces are cut within Value
    TRACEIT(10,"myFiles_SaveFileSimpleGetValue		start");
    if (Identifier == NULL){
        TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends, parameter NULL");
        return -1;
    }
    if (Value == NULL){
        TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends, parameter NULL");
        return -1;
    }

    struct myFiles_struct *stru = myFiles_openSaveFile("r");
    if (stru == NULL){
       TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends, error opening configfile");
       return -1;
    }

    if (myFiles_getValueFromIdentifier(stru, Identifier, 0) == 1){
        g_string_assign(Value, stru->gstrValue->str);
        myFiles_closeFile(stru);
        cutSpacesFromGString(Value);
        if (strlen(Value->str) == 0){
            return 0;   //not found
        }
        return 1;   //All good, found
    }
    if (stru){
        myFiles_closeFile(stru);
    }
    TRACEIT(10,"myFiles_SaveFileSimpleGetValue		ends");
    return 0;  //not found
}


struct myFiles_struct *myFiles_openFile(char *FileName, char *cpMode){
    TRACEIT(10,"myFiles_openFile		start");
    if (FileName == NULL || cpMode==NULL){
        TRACEIT(3,"myFiles_openFile		end, parameter NULL");
        //g_string_free(gstrBuf,TRUE);
        return NULL;
    }
    GString *gstrBuf = g_string_sized_new(10);

    FILE *pFile;
    pFile = fopen(FileName,cpMode);

	if (pFile == NULL){
        g_string_sprintf(gstrBuf,"myFiles_openFile		end, Error opening file %s",FileName);
        TRACEIT(3,gstrBuf->str);
        g_string_free(gstrBuf,TRUE);
        return NULL;
	}
//	openfiles++;
//	printf("Opened files: %i\n",openfiles);
    struct myFiles_struct *stru= malloc(sizeof(struct myFiles_struct));
    if (stru == NULL){
        fclose(pFile);
//        openfiles--;
//        printf("Opened files: %i\n",openfiles);
        g_string_free(gstrBuf,TRUE);
        TRACEIT(1,"myFiles_openFile		end, FATAL ERROR, MEMORYALLOCATION");
        return NULL;
    }

    stru->File = pFile;
    stru->gstrFileName = g_string_new(FileName);
    stru->gstrIdentifier = g_string_sized_new(20);
	stru->gstrValue = g_string_sized_new(20);
    stru->allreadystarted=0;

    g_string_free(gstrBuf,TRUE);
    TRACEIT(10,"myFiles_openFile		end");
    return stru;
}


int myFiles_closeFile(struct myFiles_struct *fileInfo){
    TRACEIT(10,"myFiles_closeFile		start");
    if (fileInfo == NULL){
        TRACEIT(3,"myFiles_closeFile		end fileInfo parameter == NULL");
        return -1;
    }

    if (fileInfo->File != NULL){
//        openfiles--;
//        printf("Opened files: %i\n",openfiles);
        fclose(fileInfo->File);
    }
    if (fileInfo->gstrFileName != NULL)
        g_string_free(fileInfo->gstrFileName, TRUE);
    if (fileInfo->gstrIdentifier != NULL)
        g_string_free(fileInfo->gstrIdentifier, TRUE);
    if (fileInfo->gstrValue != NULL)
        g_string_free(fileInfo->gstrValue, TRUE);

    free(fileInfo);

    TRACEIT(10,"myFiles_closeFile		end");
    return 0;
}


int myFiles_getValueFromIdentifier(struct myFiles_struct *fileInfo, char *cpIdentifier, int again){
    TRACEIT(10,"myFiles_getValueFromIdentifier		start");
    if (fileInfo == NULL){
        TRACEIT(3,"myFiles_getValueFromIdentifier		end fileInfo parameter == NULL");
        return -1;
    }
	if (cpIdentifier == NULL){
		TRACEIT(3,"myFiles_getValueFromIdentifier		ends cpIdentifier parameter == NULL");
		return -1;
	}
    if (fileInfo->File == NULL){
		TRACEIT(3,"myFiles_getValueFromIdentifier		ends No file opened");
		return -2;
	}
	char bufLine[5001];
	char *buf;
    gchar *cpctype=NULL;
    GString *gstrCType = g_string_sized_new(15);

    g_string_assign( gstrCType, cpIdentifier);
    cpctype = g_utf8_strdown( gstrCType->str, -1);  //we use only lower case letters!
    if (cpctype == NULL){
        g_string_free(gstrCType, TRUE);
        TRACEIT(10,"myFiles_getValueFromIdentifier		end");
        return 0;
    }
    g_string_assign(gstrCType, cpctype);
    g_free(cpctype);
    cpctype=NULL;

	g_string_append(gstrCType,TYPESEPARATOR);
    fileInfo->allreadystarted=0;
	if (again == 0){
		fseek(fileInfo->File,0,SEEK_SET);
	}
	while(1){
		if (feof(fileInfo->File))
			break;
		fgets(bufLine,5000,fileInfo->File);
		if ( strncmp(bufLine,gstrCType->str,strlen(gstrCType->str)) == 0 ){
		    buf =  strchr(bufLine,'\n');
			if (buf != NULL){
				*buf='\0';
			}
			g_string_assign(fileInfo->gstrValue,bufLine);
			cutTapsFromGString(fileInfo->gstrValue);
			g_string_assign(fileInfo->gstrIdentifier,gstrCType->str);
			g_string_erase(fileInfo->gstrValue,0,strlen(gstrCType->str));

			//now, in the case it is a new usercommand it is possible that the information is spread over several lines
			//this function will care of this
			//newUsercommandReadOverSeveralLines(fileInfo->File,fileInfo->gstrValue->str);
            //we decided not to do something like this, but create a program to edit the config file, then it is not
            //longer necessary to edit the configfile directly.

            g_string_free(gstrCType, TRUE);
			TRACEIT(10,"myFiles_getValueFromIdentifier		ends 1 ");
			return 1;   //found
		}
	};
	g_string_free(gstrCType, TRUE);
    TRACEIT(10,"myFiles_getValueFromIdentifier		end");
    return 0;
}


int myFiles_SeparateMultiValue(struct myFiles_struct *fileInfo, GString *name, GString *command, int ModusSingleOrPair){
	//	ModusSingleOrPair == 1 -> Pair, == 0 -> Single
	//  leading spaces are removed from command, but not from name
	TRACEIT(10,"myFiles_SeparateMultiValue		start");
	if (name == NULL || fileInfo == NULL){
		TRACEIT(10,"myFiles_SeparateMultiValue		ends");
		return -1;
	}
	if (fileInfo->gstrValue == NULL || fileInfo->gstrIdentifier == NULL){
		TRACEIT(10,"myFiles_SeparateMultiValue		ends");
		return -2;
	}

	char *buf1=NULL;
	char *buf2=NULL;
    GString *gstrCType=g_string_sized_new(15);

	const char sep[] = PROGRAMSEPERATOR;
	if (fileInfo->allreadystarted == 0){
		buf1 = (char *)strtok(fileInfo->gstrValue->str,sep);
		if (ModusSingleOrPair==1){
			buf2 = (char *)strtok(NULL,sep);
		}
		fileInfo->allreadystarted = 1;
	}
	else{
		buf1 = (char *)strtok(NULL,sep);
		if (ModusSingleOrPair==1){
			buf2 = (char *)strtok(NULL,sep);
		}
	}

	if (buf1 == NULL || (ModusSingleOrPair == 1 && buf2 == NULL)){
        g_string_assign(gstrCType, fileInfo->gstrIdentifier->str);
        g_string_truncate(gstrCType, strlen(gstrCType->str)-strlen(TYPESEPARATOR));
		if (1 == myFiles_getValueFromIdentifier(fileInfo, gstrCType->str, 1)){
			TRACEIT(10,"getOneCommandAfterEachOther		recursively ends");
			return myFiles_SeparateMultiValue(fileInfo, name, command, ModusSingleOrPair);
		}
		TRACEIT(10,"getOneCommandAfterEachOther		ends");
		return 0;	//Nothing found
	}
	else{
		if (buf1[strlen(buf1)-1]=='\n'){
			buf1[strlen(buf1)-1] = '\0';
		}
		if (ModusSingleOrPair == 1){
			if (buf2[strlen(buf2)-1]=='\n'){
				buf2[strlen(buf2)-1] = '\0';
			}
		}
		g_string_assign(name,buf1);
		if (ModusSingleOrPair == 1){
			g_string_assign(command,buf2);
			cutLeadingSpacesFromGString(command);
		}

		TRACEIT(10,"getOneCommandAfterEachOther		ends");
		return 1;   //found
	}
}


int myFiles_findStringInFile(struct myFiles_struct *fileInfo, char *cpString){
    //this will NOT rewind the file !!
    //return: -1=error, 0=notfound, 1=found
    TRACEIT(10,"myFiles_findStringInFile		start");
    if (fileInfo==NULL || cpString==NULL){
        TRACEIT(2,"myFiles_findStringInFile		end, Parameter NULL");
        return -1;
    }
    if (fileInfo->File == NULL){
        TRACEIT(2,"myFiles_findStringInFile		end, Parameter 2 NULL");
        return -1;
    }

    char bufLine[5001];

    while(1){
		if (feof(fileInfo->File))
			break;
		fgets(bufLine,5000,fileInfo->File);
		if ( strstr( bufLine, cpString ) != NULL ){

			TRACEIT(10,"myFiles_getValueFromIdentifier		ends 1 ");
			return 1;   //found
		}
	}

    TRACEIT(10,"myFiles_findStringInFile		end");
    return 0;   //not found
}


int myFiles_SaveValueToFile(struct myFiles_struct *fileInfo, char* cpLine){
    TRACEIT(10,"myFiles_SaveValueToFile		start");
    if (fileInfo == NULL){
        TRACEIT(3,"myFiles_SaveValueToFile		end fileInfo parameter == NULL");
        return -1;
    }
	if (cpLine == NULL){
		TRACEIT(3,"myFiles_SaveValueToFile		ends cpLine parameter == NULL");
		return -1;
	}
	if (fileInfo->File == NULL){
		TRACEIT(3,"myFiles_SaveValueToFile		ends No file opened");
		return -2;
	}

    fputs(cpLine,fileInfo->File);
	fputc('\n',fileInfo->File);

    TRACEIT(10,"myFiles_SaveValueToFile		ends");
    return 0;
}

